SystemOrganization addCategory: #Trachel!
SystemOrganization addCategory: 'Trachel-Core'!
SystemOrganization addCategory: 'Trachel-Viva'!

Morph subclass: #TRMorph
	instanceVariableNames: 'trachelCanvas surface eventBeginingDragging shapeBeingPointed shouldRefresh session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRMorph methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:29'!
canvas: aTrachelCanvas
	trachelCanvas := aTrachelCanvas.
	surface := AthensCairoSurface extent: aTrachelCanvas extent! !

!TRMorph methodsFor: 'session management' stamp: 'AlexandreBergel 1/12/2014 15:37'!
checkSession
	session == Smalltalk session
		ifFalse: [ surface := nil. self createSurface ]! !

!TRMorph methodsFor: 'session management' stamp: 'AlexandreBergel 1/12/2014 16:47'!
createSurface
"	surface isNil ifFalse: [ ^ self ]."
	surface := AthensCairoSurface extent: self extent asIntegerPoint.
	session := Smalltalk session.! !

!TRMorph methodsFor: 'drawing' stamp: 'AlexandreBergel 1/12/2014 15:43'!
"protocol: drawing"
drawOn: aCanvas
	"aCanvas is a FormCanvas"
	| offset |
	
"	self checkSession."
	
	aCanvas fillRectangle: bounds color: trachelCanvas color.
	trachelCanvas playAnimations.
	
	"The drawing has to be done when a change in the shapes occured or when there is an animation."
	surface drawDuring: [:cs |
		surface clear.
		
		"We display the elements that are subject to the camera"
		offset := (self extent / 2) - trachelCanvas camera position.
		cs pathTransform translateX: offset x Y: offset y.
		cs pathTransform 
			scaleBy: trachelCanvas camera scale.
		trachelCanvas shapes do: [ :trachelShape |
			trachelShape drawOn: cs.
		].
	
		"We display the elements that are _NOT_ subject to the camera"
		cs pathTransform loadIdentity.
		trachelCanvas fixedShapes do: [ :trachelShape |
			trachelShape drawOn: cs.
		].
	].
	"aCanvas translucentImage: surface asForm at: self bounds origin."
	"asForm creates a new Form, which is likely to be expensive. This can be cached"
	aCanvas image: surface asForm at: self bounds origin sourceRect: (0 @ 0 extent: surface extent) rule: 34! !

!TRMorph methodsFor: 'events-processing' stamp: 'AlexandreBergel 12/15/2013 19:44'!
handleMouseMove: anEvent
	super handleMouseMove: anEvent.
	self rtMouseMoving: anEvent.! !

!TRMorph methodsFor: 'events-processing' stamp: 'AlexandreBergel 12/16/2013 15:45'!
handleMouseOver: anEvent

	| currentElement event |
	anEvent wasHandled ifTrue:[ ^ self ]. "not interested"
	"Do nothing if we are dragging"
	self isDragging ifTrue: [ ^ self ].
	 
	shapeBeingPointed ifNil: [ shapeBeingPointed := self shapeForEvent: anEvent ].
	
	currentElement := self shapeForEvent: anEvent.
	(currentElement ~~ shapeBeingPointed)
		ifTrue: [ 
			event := TRMouseLeave new.
			event morph: self.
			event position: (self relativePositionFor: anEvent).
			event shape: shapeBeingPointed.

			shapeBeingPointed announce: event.
			shapeBeingPointed := currentElement.
			self rtMouseEnter: anEvent ].
! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/15/2013 19:35'!
handlesMouseDown: anEvent
	anEvent wasHandled ifTrue:[ ^ false ]. "not interested"

	^ true! !

!TRMorph methodsFor: 'morphic' stamp: 'AlexandreBergel 12/15/2013 19:34'!
handlesMouseOver: evt
	^ true! !

!TRMorph methodsFor: 'initialization' stamp: 'AlexandreBergel 1/12/2014 15:43'!
initialize
	super initialize.
	self createSurface.
	shouldRefresh := false.
	session := Smalltalk session.! !

!TRMorph methodsFor: 'testing' stamp: 'AlexandreBergel 12/15/2013 19:46'!
isDragging
	^ eventBeginingDragging notNil! !

!TRMorph methodsFor: 'morphic' stamp: 'AlexandreBergel 12/29/2013 13:37'!
layoutChanged 
	"react on morph resize"
	super layoutChanged.
	surface ifNotNil: [  
		self extent asIntegerPoint ~= surface extent ifTrue: [ 
			self createSurface.
			trachelCanvas extent: self extent ]
	].
	! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/16/2013 15:57'!
mouseDown: evt

	eventBeginingDragging ifNil: [
		evt hand 
					waitForClicksOrDrag: self 
					event: evt 
					selectors: { #rtMouseClick:. nil. nil. #rtMouseDragBegin: }
					threshold: 5.
		^ self ].
	
	evt anyButtonPressed ifTrue: [
		self rtMouseDragging: evt.
	].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/15/2013 23:08'!
mouseEnter: evt
	evt anyButtonPressed ifFalse: [
		self rtMouseEnter: evt.
	].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/15/2013 19:34'!
mouseLeave: evt
	
	evt anyButtonPressed ifFalse: [
		self rtMouseLeave: evt.
	].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/16/2013 15:58'!
mouseMove: evt
	
	evt anyButtonPressed
		ifTrue: [ self rtMouseDragging: evt ] 
		ifFalse: [ self rtMouseMoving: evt ].
	evt wasHandled: true.
	^ true! !

!TRMorph methodsFor: 'event handling' stamp: 'AlexandreBergel 12/16/2013 16:05'!
mouseUp: evt
	self isDragging ifTrue: 
		[self rtMouseDragEnd: evt ].
	evt wasHandled: true.! !

!TRMorph methodsFor: 'events-roassal' stamp: 'AlexandreBergel 12/31/2013 10:42'!
relativePositionFor: evt
	"Return the position within the window"
	^ evt position - self bounds origin
	"^ trachelCanvas camera fromPixelToSpace: (evt position - self bounds origin) "! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 1/17/2014 10:42'!
rtMouseClick: evt
	| relativePosition ev  shape |
	relativePosition := self relativePositionFor: evt.	
	evt yellowButtonChanged
		ifTrue: [ ev := TRMouseRightClick new ].
	evt redButtonChanged
		ifTrue: [ ev := TRMouseLeftClick new ].
	evt blueButtonChanged
		ifTrue: [ ev := TRMouseClick new ].

	shape := self shapeForPositionInPixels: relativePosition.
	ev commandKeyPressed: evt commandKeyPressed.
	ev controlKeyPressed: evt controlKeyPressed.
	ev shiftKeyPressed: evt shiftPressed.
	ev shape: shape.
	ev morph: self.
	
	ev position: (trachelCanvas camera fromPixelToSpace: relativePosition).
	shape announce: ev.
	! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseDragBegin: evt
	| relativePosition |
	eventBeginingDragging := evt copy.
	
	relativePosition := self relativePositionFor: evt.
	shapeBeingPointed := self shapeForPositionInPixels: relativePosition! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/16/2013 16:05'!
rtMouseDragEnd: evt
	eventBeginingDragging := nil.
	shapeBeingPointed := nil! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseDragging: evt
	| step relativePosition event  |
	eventBeginingDragging ifNil: [ ^ self ].
	relativePosition := self relativePositionFor: evt.
	step := evt position - eventBeginingDragging position.

	shapeBeingPointed ifNil: 
		[ shapeBeingPointed := self shapeForPositionInPixels: relativePosition ].
	
	event := TRMouseDragging new 
				morph: self;
				shape: shapeBeingPointed;
				step: step; 
				yourself.

"	event commandKeyPressed: evt commandKeyPressed.
	event controlKeyPressed: evt controlKeyPressed.
	event shiftKeyPressed: evt shiftPressed.				
"
	event position: relativePosition.
	shapeBeingPointed announce: event.

	eventBeginingDragging := evt copy.! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseEnter: evt
	| relativePosition ev shape |
	
	relativePosition := self relativePositionFor: evt.

	shape := self shapeForPositionInPixels: relativePosition.
	ev := TRMouseEnter new.
	ev position: relativePosition.
	ev shape: shape.
	ev morph: self.
	shape announce: ev.
! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:31'!
rtMouseLeave: evt
	| position ev shape |
	
	position := self relativePositionFor: evt.
	shape := self shapeForPositionInPixels: position.
	ev := TRMouseLeave new.
	ev position: position.
	ev shape: shape.
	ev morph: self.
	shape announce: ev
! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:32'!
rtMouseMoving: evt

	| relativePosition ev shape |
	relativePosition := self relativePositionFor: evt.	
	shape := self shapeForPositionInPixels: relativePosition.

	ev := TRMouseMove new.
	ev position: relativePosition.
	ev shape: shape.
	ev morph: self.
	shape announce: ev! !

!TRMorph methodsFor: 'event-processed' stamp: 'AlexandreBergel 12/31/2013 10:32'!
shapeForEvent: anEvent	
	^ self shapeForPositionInPixels: (self relativePositionFor: anEvent)! !

!TRMorph methodsFor: 'accessing' stamp: 'AlexandreBergel 12/31/2013 10:31'!
shapeForPositionInPixels: point
	"Return the shape located that contains point"
	 ^ trachelCanvas shapeForPositionInPixels: point! !

!TRMorph methodsFor: 'signalling' stamp: 'AlexandreBergel 12/25/2013 09:28'!
signalUpdate
	self changed! !

!TRMorph methodsFor: 'stepping' stamp: 'AlexandreBergel 12/25/2013 08:39'!
step
	self changed! !

!TRMorph methodsFor: 'stepping' stamp: 'AlexandreBergel 12/23/2013 18:59'!
stepTime
	"Answer the desired time between steps in milliseconds. This default implementation requests that the 'step' method be called once every second."

	^ 20! !

!TRMorph methodsFor: 'testing' stamp: 'AlexandreBergel 12/18/2013 23:20'!
wantsSteps
	^ trachelCanvas hasAnimation ! !

Announcement subclass: #TREvent
	instanceVariableNames: 'canvas position morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
canvas
	
	^ canvas! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
canvas: aTrachelCanvas
	
	canvas := aTrachelCanvas! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 23:44'!
morph
	^ morph! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:31'!
morph: aMorph
	morph := aMorph! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
position
	
	^ position! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/15/2013 22:56'!
position: aPoint
	
	position := aPoint! !

!TREvent methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:32'!
signalUpdate
	morph notNil ifTrue: [ morph changed ]
	! !

TREvent subclass: #TRResizeCanvasEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TREvent subclass: #TRShapeEvent
	instanceVariableNames: 'shape'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRShapeEvent subclass: #TRMouseClick
	instanceVariableNames: 'commandKeyPressed controlKeyPressed shiftKeyPressed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRMouseClick commentStamp: '' prior: 0!
A ROMouseClick is xxxxxxxxx.

Instance Variables
	commandKeyPressed:		<Object>
	controlKeyPressed:		<Object>
	shiftKeyPressed:		<Object>

commandKeyPressed
	- xxxxx

controlKeyPressed
	- xxxxx

shiftKeyPressed
	- xxxxx
!

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
commandKeyPressed
	^ commandKeyPressed! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
commandKeyPressed: anObject
	commandKeyPressed := anObject! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
controlKeyPressed
	^ controlKeyPressed! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
controlKeyPressed: anObject
	controlKeyPressed := anObject! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
initialize 
	super initialize.
	commandKeyPressed := false.
	controlKeyPressed := false.
	shiftKeyPressed := false.! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
shiftKeyPressed
	^ shiftKeyPressed! !

!TRMouseClick methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 19:08'!
shiftKeyPressed: anObject
	shiftKeyPressed := anObject! !

TRMouseClick subclass: #TRMouseLeftClick
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRMouseClick subclass: #TRMouseRightClick
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRShapeEvent subclass: #TRMouseDragging
	instanceVariableNames: 'step'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRMouseDragging class methodsFor: 'public' stamp: 'AlexandreBergel 12/31/2013 09:05'!
step: aStepAsPoint
	^ self new step: aStepAsPoint! !

!TRMouseDragging methodsFor: 'initialization' stamp: 'AlexandreBergel 12/16/2013 15:51'!
initialize
	super initialize.
	step := 0 @ 0! !

!TRMouseDragging methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:51'!
step
	^ step! !

!TRMouseDragging methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 15:51'!
step: aPoint
	step := aPoint! !

TRShapeEvent subclass: #TRMouseEnter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRShapeEvent subclass: #TRMouseLeave
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRShapeEvent subclass: #TRMouseMove
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRShapeEvent class methodsFor: 'instance creation' stamp: 'JurajKubelka 1/27/2014 11:22'!
shape: aTrachelShape
	^ self new shape: aTrachelShape; yourself! !

!TRShapeEvent methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 17:11'!
element
	"Could be any object"
	^ self shape element! !

!TRShapeEvent methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 17:10'!
shape
	
	^ shape! !

!TRShapeEvent methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 17:10'!
shape: aTrachelShape
	
	shape := aTrachelShape! !

!Point methodsFor: '*Trachel' stamp: 'AlexandreBergel 1/24/2014 19:20'!
relativeAngleTo: aPoint
	| width height hip |
	height:= (self y - aPoint y) abs.
	width:= (self x -aPoint x) abs.
	(height = 0 and: [ width = 0 ]) ifTrue: [ ^ 0 ].
	hip:= (self dotProduct: aPoint) sqrt.
	
	"First quadrant"
	(self x > 0 and: [self y > 0]) ifTrue: [ (width @ height) angle]
	
	
	! !

TestCase subclass: #TRTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRTest subclass: #TRArcShapeTest
	instanceVariableNames: 'arc1 arc2 arc3 arc4 arc5 p1 p2 p3 p4 p5 p6 p7 p8 p1n p2n p3n p4n p5n p6n p7n p8n arcQ1 arcQ2 arcQ3 arcQ4 pR pB pL pT pQ1 pQ2 pQ3 pQ4'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRArcShapeTest commentStamp: '<historical>' prior: 0!
A TRArcShapeTest is a test class for testing the behavior of TRArcShape!

!TRArcShapeTest methodsFor: 'basic tests' stamp: 'SamirSaleh 1/28/2014 10:49'!
setUp
	arc1:=TRArcShape new.
	arc2:=TRArcShape new.
	arc3:=TRArcShape new.
	arc4:=TRArcShape new.
	arc5:=TRArcShape new.
	arcQ1:=TRArcShape new.
	arcQ2:=TRArcShape new.
	arcQ3:=TRArcShape new.
	arcQ4:=TRArcShape new.
	arc1 alphaAngle: 0 betaAngle: 360 innerRadius:  0 externalRadius:  100.
	arc2 alphaAngle: 30 betaAngle: 60 innerRadius:  50 externalRadius:  50.
	arc3 alphaAngle: 0 betaAngle:  90 innerRadius:  2 sqrt externalRadius: 2 sqrt.
	arc4 alphaAngle:180 betaAngle: 270 innerRadius: 50 externalRadius:150.
	arc5 alphaAngle:0 betaAngle: 270 innerRadius: 50 externalRadius:150.
	arcQ1 alphaAngle: 0 betaAngle: 90 innerRadius:  0 externalRadius:  100.
	arcQ2 alphaAngle: 90 betaAngle: 180 innerRadius:  0 externalRadius:  100.
	arcQ3 alphaAngle: 180 betaAngle: 270 innerRadius:  0 externalRadius:  100.
	arcQ4 alphaAngle: 270 betaAngle: 360 innerRadius:  0 externalRadius:  100.
	pR:=100 @ 0.
	pB:=0 @ 100.
	pL:=-100 @ 0.
	pT:=0 @ -100.
	pQ1:=50 @ 50.
	pQ2:=-50 @ 50.
	pQ3:=-50 @ -50.
	pQ4:=50 @ -50.
	p1:= 0 @ 0.
	p2:=100 @ 100.
	p3:=50 @ 50.
	p4:= 0 @ 150.
	p5:=1 @ 1.
	p6:=0 @ -100.
	p7:=-100 @ 0.
	p8:=-100 @ -100.
	p1n:=p1 negated.
	p2n:=p2 negated.
	p3n:=p3 negated.
	p4n:= p4 negated.
	p5n:=p5 negated.
	p6n:=p6 negated.
	p7n:=p7 negated.
	p8n:=p8 negated.! !

!TRArcShapeTest methodsFor: 'basic tests' stamp: 'SamirSaleh 1/27/2014 11:22'!
testEncompassingRectangle
	self assert: (arc1 encompassingRectangle = ((-100.0@100.0) corner: (100.0@ -100.0))).
	self assert: (arc1 encompassingRectangle width abs = 200).
	self assert:(arc2 encompassingRectangle width abs < 50).  
	self assert:(arc2 encompassingRectangle height abs < 50).
	self assert: (arc3 encompassingRectangle height abs= 2 sqrt).
	self assert: (((arc4 encompassingRectangle height abs) roundDownTo: 1 )= 150). 
	self assert: (((arc4 encompassingRectangle width abs) roundDownTo: 1 )= 150). 
	self assert: (arc5 encompassingRectangle width abs= 300).
	self assert: (arc5 encompassingRectangle height abs= 300).! !

!TRArcShapeTest methodsFor: 'basic tests' stamp: 'SamirSaleh 1/28/2014 10:49'!
testIncludesPoint	
	self assert: (arc1 includesPoint: p1).
	self deny: (arc1 includesPoint: p2).
	self assert: (arc1 includesPoint: p3).
	self deny: (arc1 includesPoint: p4).
	self assert: (arc1 includesPoint: p5).	
	self assert: (arc1 includesPoint: p6).
	self assert: (arc1 includesPoint: p7).
	self deny: (arc1 includesPoint: p8).
	self assert: (arc1 includesPoint: p1n).
	self deny: (arc1 includesPoint: p2n).
	self assert: (arc1 includesPoint: p3n).
	self deny: (arc1 includesPoint: p4n).
	self assert: (arc1 includesPoint: p5n).	
	self assert: (arc1 includesPoint: p6n).
	self assert: (arc1 includesPoint: p7n).
	self deny: (arc1 includesPoint: p8n).
	
	self deny: (arc2 includesPoint: p1).
	self deny: (arc2 includesPoint: p2).
	self deny: (arc2 includesPoint: p3).
	self deny: (arc2 includesPoint: p4).
	self deny: (arc2 includesPoint: p5).	
	self deny: (arc2 includesPoint: p6).
	self deny: (arc2 includesPoint: p7).
	self deny: (arc2 includesPoint: p8).
	self deny: (arc2 includesPoint: p1n).
	self deny: (arc2 includesPoint: p2n).
	self deny: (arc2 includesPoint: p3n).
	self deny: (arc2 includesPoint: p4n).
	self deny: (arc2 includesPoint: p5n).	
	self deny: (arc2 includesPoint: p6n).
	self deny: (arc2 includesPoint: p7n).
	self deny: (arc2 includesPoint: p8n).
	
	self deny: (arc3 includesPoint: p1).
	self deny: (arc3 includesPoint: p2).
	self deny: (arc3 includesPoint: p3).
	self deny: (arc3 includesPoint: p4).
	self assert: (arc3 includesPoint: p5).
	self deny: (arc3 includesPoint: p6).
	self deny: (arc3 includesPoint: p7).
	self deny: (arc3 includesPoint: p8).
	self deny: (arc3 includesPoint: p1n).
	self deny: (arc3 includesPoint: p2n).
	self deny: (arc3 includesPoint: p3n).
	self deny: (arc3 includesPoint: p4n).
	self deny: (arc3 includesPoint: p5n).
	self deny: (arc3 includesPoint: p6n).
	self deny: (arc3 includesPoint: p7n).
	self deny: (arc3 includesPoint: p8n).
	
	self deny: (arc4 includesPoint: p1).
	self deny: (arc4 includesPoint: p2).
	self deny: (arc4 includesPoint: p3).
	self deny: (arc4 includesPoint: p4).
	self deny: (arc4 includesPoint: p5).	
	self assert:(arc4 includesPoint: p6).
	self assert:(arc4 includesPoint: p7).
	self assert:(arc4 includesPoint: p8).
	self deny: (arc4 includesPoint: p1n).
	self assert: (arc4 includesPoint: p2n).
	self assert: (arc4 includesPoint: p3n).
	self assert: (arc4 includesPoint: p4n).
	self deny: (arc4 includesPoint: p5n).	
	self deny: (arc4 includesPoint: p6n).
	self deny: (arc4 includesPoint: p7n).
	self deny: (arc4 includesPoint: p8n).
	
	self deny: (arc5 includesPoint: p1).
	self assert: (arc5 includesPoint: p2).
	self assert: (arc5 includesPoint: p3).
	self assert: (arc5 includesPoint: p4).
	self deny: (arc5 includesPoint: p5).	
	self assert:(arc5 includesPoint: p6).
	self assert:(arc5 includesPoint: p7).
	self assert:(arc5 includesPoint: p8).
	self deny: (arc5 includesPoint: p1n).
	self assert: (arc5 includesPoint: p2n).
	self assert: (arc5 includesPoint: p3n).
	self assert: (arc5 includesPoint: p4n).
	self deny: (arc5 includesPoint: p5n).	
	self assert: (arc5 includesPoint: p6n).
	self assert: (arc5 includesPoint: p7n).
	self assert: (arc5 includesPoint: p8n).
	
	self assert:(arcQ1 includesPoint: pR).
	self assert:(arcQ1 includesPoint: pQ1).
	self assert:(arcQ1 includesPoint: pB).
	self deny:(arcQ1 includesPoint: pQ2).
	self deny:(arcQ1 includesPoint: pL).
	self deny:(arcQ1 includesPoint: pQ3).
	self deny:(arcQ1 includesPoint: pT).
	self deny:(arcQ1 includesPoint: pQ4).
	
	self deny:(arcQ2 includesPoint: pR).
	self deny:(arcQ2 includesPoint: pQ1).
	self assert:(arcQ2 includesPoint: pB).
	self assert:(arcQ2 includesPoint: pQ2).
	self assert:(arcQ2 includesPoint: pL).
	self deny:(arcQ2 includesPoint: pQ3).
	self deny:(arcQ2 includesPoint: pT).
	self deny:(arcQ2 includesPoint: pQ4).
	
	self deny:(arcQ3 includesPoint: pR).
	self deny:(arcQ3 includesPoint: pQ1).
	self deny:(arcQ3 includesPoint: pB).
	self deny:(arcQ3 includesPoint: pQ2).
	self assert:(arcQ3 includesPoint: pL).
	self assert:(arcQ3 includesPoint: pQ3).
	self assert:(arcQ3 includesPoint: pT).
	self deny:(arcQ3 includesPoint: pQ4).
	
	self assert:(arcQ4 includesPoint: pR).
	self deny:(arcQ4 includesPoint: pQ1).
	self deny:(arcQ4 includesPoint: pB).
	self deny:(arcQ4 includesPoint: pQ2).
	self deny:(arcQ4 includesPoint: pL).
	self deny:(arcQ4 includesPoint: pQ3).
	self assert:(arcQ4 includesPoint: pT).
	self assert:(arcQ4 includesPoint: pQ4).
! !

TRTest subclass: #TRCallbackTest
	instanceVariableNames: 'canvas circle1 circle2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRCallbackTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/29/2013 10:18'!
setUp

	canvas := TRCanvas new.
	circle1 := canvas buildCircleFrom: 20 @ 20 to: 150 @ 100 color: (Color green alpha: 0.1).
	circle2 := canvas buildCircleFrom: 200 @ 200 to: 240 @ 230 color: (Color blue alpha: 0.1).	! !

!TRCallbackTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/29/2013 11:01'!
testCallback
	| callback v s  |
	callback := TRTranslationCallback 
						block: [ :shape :step | 
									v := shape. s := step ].
	circle1 addCallback: callback.

	circle1 translateTo: 20 @ 30.
	self assert: v == circle1.
	self assert: v center = (20 @ 30).
	self assert: s = (-65 @ -30).

	v := nil.
	circle1 translateBy: 5 @ 3.
	self assert: v == circle1.
	self assert: v center = (25 @ 33).
	self assert: s = (5 @ 3).
	! !

!TRCallbackTest methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 12/29/2013 10:26'!
testHasCallBack

	self deny: TREllipseShape new hasCallback.
	self assert: (TREllipseShape new addCallback: TRTranslationCallback new) hasCallback! !

TRTest subclass: #TRCameraTest
	instanceVariableNames: 'camera canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRCameraTest methodsFor: 'running' stamp: 'AlexandreBergel 12/29/2013 13:21'!
setUp
	super setUp. 
	canvas := TRCanvas new.
	camera := canvas camera! !

!TRCameraTest methodsFor: 'conversion' stamp: 'AlexandreBergel 12/29/2013 13:29'!
testConversion
	self assert: (camera fromSpaceToPixel: 0 @ 0) = (canvas extent / 2).
	self assert: (camera fromSpaceToPixel: 10 @ 10) = ((canvas extent / 2) + (10 @ 10)).

	self assert: (camera fromPixelToSpace: (canvas extent / 2)) = (0 @ 0).
	self assert: (camera fromPixelToSpace: ((canvas extent / 2) + (10 @ 10))) = (10 @ 10)! !

!TRCameraTest methodsFor: 'focusing' stamp: 'AlexandreBergel 12/29/2013 13:21'!
testFocusingCenter
	canvas addShape: (TREllipseShape new from: 50 @ 50 to: 100 @ 80).
	canvas camera focusOnCenter.
	self assert: canvas camera position = (75 @ 65)! !

!TRCameraTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 11:39'!
testInitialization
	self assert: camera position = (0 @ 0)! !

!TRCameraTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 11:40'!
testTranslation
	camera translateTo: 5 @ 3.
	self assert: camera position = (5 @ 3).
	
	camera translateBy: 2 @ 4.
	self assert: camera position = (7 @ 7).! !

TRTest subclass: #TRCanvasTest
	instanceVariableNames: 'canvas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRCanvasTest methodsFor: 'running' stamp: 'AlexandreBergel 12/14/2013 11:06'!
setUp

	canvas := TRCanvas new! !

!TRCanvasTest methodsFor: 'fixed shape' stamp: 'AlexandreBergel 12/31/2013 10:39'!
testAccessing
	| c r1 r2 |
	c := TRCanvas new.
	r1 := c buildRectangle: ((0 @ 0) extent: (150 @ 75)) color: Color red.
	r1 setAsFixed.
	
	r2 := c buildRectangle: ((0 @ 0) extent: (130 @ 50)) color: Color lightGreen.

	self assert: (c shapeForPositionInPixels: (0 @ 0)) = r1.
	self assert: (c shapeForPositionInPixels: (250 @ 250)) = r2.! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 1/5/2014 18:52'!
testAddingAndRemoving

	| box |
	box := TRBoxShape new from: 10 @ 10 to: 20 @ 20.
	canvas addShape: box.
	self assert: canvas numberOfShapes = 1.
	box remove.
	self assert: canvas numberOfShapes = 0.! !

!TRCanvasTest methodsFor: 'back and front' stamp: 'AlexandreBergel 1/8/2014 15:21'!
testBack

	| s1 s2 s3 |
	s1 := TRBoxShape new size: 10.
	s2 := TRBoxShape new size: 10.
	s3 := TRBoxShape new size: 10.
	canvas addShape: s1; addShape: s2; addShape: s3.
	
	s1 pushBack.
	self assert: canvas shapes = (Array with: s1 with: s2 with: s3).
	
	s3 pushBack.
	self assert: canvas shapes = (Array with: s3 with: s1 with: s2).! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 1/16/2014 15:32'!
testBezierShape
	| b |
	b := TRBezierShape new points: (Array with: 10 @ 10 with: -20 @ 30 with: 50 @ -180).
	canvas addShape: b.
	b translateBy: 5 @ 6.
	self assert: b points = (Array with: 15 @ 16 with: -15 @ 36 with: 55 @ -174).
! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/24/2013 15:47'!
testBox
	| rec box |
	rec := (50 @ 50 extent: 100 @ 50).
	box := canvas buildRectangle: rec color: Color purple.
	self assert: box rectangle center = rec center.

	box extent: 20 @ 20.
	self assert: box rectangle center = rec center.! !

!TRCanvasTest methodsFor: 'camera' stamp: 'AlexandreBergel 12/29/2013 11:48'!
testCamera

	self assert: canvas camera position = (0 @ 0)! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/25/2013 09:25'!
testCanvas
	| box |
	box := canvas buildRectangle: ( (5 @ 5) extent: (5 @ 5) ) color: Color red.
	self assert: box canvas == canvas.
	self assert: box athensCanvas == canvas athensCanvas.
	self assert: canvas ~~ canvas athensCanvas.! !

!TRCanvasTest methodsFor: 'events' stamp: 'JurajKubelka 1/27/2014 10:22'!
testCanvasAndEvent

	| c f |
	c := TRCanvas new.
	f := false.
	c when: TRMouseClick do: [ :event | f := true ].
	self deny: f.
	c announce: TRMouseMove.
	self deny: f.
	c announce: TRMouseClick.
	self assert: f! !

!TRCanvasTest methodsFor: 'events' stamp: 'AlexandreBergel 1/8/2014 15:13'!
testCapturingEvents2
	| s1 s2 |
	 
	s1 := (TRBoxShape new size: 10).
	s2 := (TRBoxShape new size: 10).
	canvas addShape: s1.
	canvas addShape: s2. 
	self assert: (canvas shapeForPosition: 0 @ 0) == s2! !

!TRCanvasTest methodsFor: 'label' stamp: 'AlexandreBergel 1/1/2014 14:12'!
testCenteredLabelShapes

	| label1 label2  |
	label1 := TRLabelShape new.
	label1 text: 'hello'.
	label1 position: 10 @ 10.

	label2 := TRLabelShape new.
	label2 text: 'hello'.
	label2 leftPosition: label1 encompassingRectangle leftCenter.
	
	"
	TRCanvas new addShape: label1; addShape: label2; open
	"

	self assert: label1 encompassingRectangle = label2 encompassingRectangle! !

!TRCanvasTest methodsFor: 'shapes' stamp: 'AlexandreBergel 1/23/2014 16:51'!
testChangingBoxHeight
	| box |
	box := TRBoxShape new.
	self assert: box height = 0.
	box height: 15.
	self assert: box height = 15.
	box height: 20.
	self assert: box height = 20
! !

!TRCanvasTest methodsFor: 'shapes' stamp: 'AlexandreBergel 1/23/2014 17:00'!
testChangingBoxWidth
	| box |
	box := TRBoxShape new.
	self assert: box width = 0.
	box width: 15.
	self assert: box width = 15.
	box width: 20.
	self assert: box width = 20
! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/24/2013 18:39'!
testCircle

	| c  |
	c := TRCanvas new.
	100 to: 200 by: 20 do: [ :i |
		 c buildCircleFrom: 50 @ 50 to: i @ 100 color: (Color red alpha: 0.2).
	].
	self assert: c numberOfShapes = (100 to: 200 by: 20) size ! !

!TRCanvasTest methodsFor: 'composing' stamp: 'AlexandreBergel 1/14/2014 15:25'!
testComposing

	| s1 s2 c trCanvas |
	s1 := TRBoxShape new.
	s2 := TRBoxShape new.
	c := TRComposedShape with: s1 with: s2.
	trCanvas := TRCanvas new.
	trCanvas addShape: c.
	self assert: trCanvas numberOfShapes = 2.
	
	c remove.
	self assert: trCanvas numberOfShapes = 0.! !

!TRCanvasTest methodsFor: 'contains' stamp: 'AlexandreBergel 12/15/2013 19:07'!
testContainsPoint1
	self assert: (canvas shapeForPosition: 20 @ 20) == canvas! !

!TRCanvasTest methodsFor: 'contains' stamp: 'AlexandreBergel 12/15/2013 19:17'!
testContainsPoint2
	| box |
	box := canvas buildRectangle: (15 @ 15 extent: 20 @ 20) color: Color gray.
	self assert: (canvas shapeForPosition: 20 @ 20) == box.
	self assert: (canvas shapeForPosition: 35 @ 35) == canvas.
	self assert: (canvas shapeForPosition: 15 @ 15) == box.
	self assert: (canvas shapeForPosition: 200 @ 200) == canvas.! !

!TRCanvasTest methodsFor: 'contains' stamp: 'AlexandreBergel 12/29/2013 10:50'!
testContainsPointAfterTranslation
	| box |
	box := canvas buildRectangle: (0 @ 0 extent: 20 @ 20) color: Color gray.

	box translateBy: 10 @ 10.
	
	self assert: (canvas shapeForPosition: 10 @ 10) == box.
	self assert: (canvas shapeForPosition: 5 @ 5) == canvas.
! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/14/2013 18:52'!
testDefault

	self assert: canvas shapes isEmpty! !

!TRCanvasTest methodsFor: 'animations' stamp: 'AlexandreBergel 12/18/2013 18:26'!
testDefaultAnimation
	self deny: canvas hasAnimation.
	self assert: canvas numberOfAnimations = 0! !

!TRCanvasTest methodsFor: 'label' stamp: 'AlexandreBergel 1/1/2014 18:11'!
testDefaultFontSize
	self assert: (TRLabelShape new fontSize = TRLabelShape new defaultFontSize)! !

!TRCanvasTest methodsFor: 'fixed shape' stamp: 'AlexandreBergel 12/31/2013 10:16'!
testFixedShapes

	| s1 s2 |
	s1 := TRBoxShape new from: 10 @ 20 extent: 50 @ 40.
	s2 := TRBoxShape new from: 10 @ 20 extent: 50 @ 40.
	canvas addShape: s1; addShape: s2.
	
	self assert: s1 isNotFixed.
	self assert: s2 isNotFixed.
	
	s2 setAsFixed.
	self assert: s1 isNotFixed.
	self deny: s2 isNotFixed.
	
	s2 setAsNotFixed.
	self assert: s1 isNotFixed.
	self assert: s2 isNotFixed.
! !

!TRCanvasTest methodsFor: 'back and front' stamp: 'AlexandreBergel 1/8/2014 15:21'!
testFront

	| s1 s2 s3 |
	s1 := TRBoxShape new size: 10.
	s2 := TRBoxShape new size: 10.
	s3 := TRBoxShape new size: 10.
	canvas addShape: s1; addShape: s2; addShape: s3.
	
	self assert: canvas shapes = (Array with: s1 with: s2 with: s3).
	
	s1 pushFront.
	self assert: canvas shapes = (Array with: s2 with: s3 with: s1).
	
	s3 pushFront.
	self assert: canvas shapes = (Array with: s2 with: s1 with: s3).! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 11:14'!
testInitialization

	| circle |
	circle := TREllipseShape new.
	self assert: circle rectangle = TRAbstractBoxShape nullRectangle! !

!TRCanvasTest methodsFor: 'label' stamp: 'AlexandreBergel 12/29/2013 22:17'!
testLabelShapes

	| label1 label2 rec |
	label1 := TRLabelShape new.
	label1 text: 'hello'.
	label2 := TRLabelShape new.
	label2 text: 'hello world'.
	self assert: label2 width > label1 width.
	
	rec := label1 encompassingRectangle.
	self assert: rec center = (0 @ 0).! !

!TRCanvasTest methodsFor: 'label' stamp: 'AlexandreBergel 12/31/2013 11:11'!
testLabelShapesAndColor

	| label  |
	label := TRLabelShape new.
	label color: Color blue.
	self assert: label color = Color blue! !

!TRCanvasTest methodsFor: 'label' stamp: 'AlexandreBergel 12/31/2013 11:06'!
testLabelShapesAndIncludesPoint

	| label rec |
	label := TRLabelShape new.
	label text: 'hello'.
	
	rec := label encompassingRectangle.
	self assert: (label includesPoint: rec topLeft).
	self assert: (label includesPoint: (rec bottomLeft - (0 @ 1))).
	self assert: (label includesPoint: rec topRight - (1 @ 0)).
	self assert: (label includesPoint: rec bottomRight - (1 @ 1)).! !

!TRCanvasTest methodsFor: 'label' stamp: 'AlexandreBergel 12/29/2013 22:05'!
testLabelShapesHeight

	| label1 label2 |
	label1 := TRLabelShape new.
	label1 text: 'hello'.
	label2 := TRLabelShape new.
	label2 text: 'hello world'.
	self assert: label2 height = label1 height! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 10:31'!
testLine
	| line |
	line := canvas buildLineFrom: 2 @ 3 to: 13 @ 15 color: Color blue.
	self assert: line from = (2 @ 3).
	self assert: line to = (13 @ 15).
	
	line translateBy: 3 @ 4.
	self assert: line from = (5 @ 7).
	self assert: line to = (16 @ 19).
! !

!TRCanvasTest methodsFor: 'shapes' stamp: 'AlexandreBergel 1/22/2014 15:45'!
testPositionWithShapes

	TRShape allSubclasses do: [ :c |
		self assert: c new position = (0 @ 0) ]! !

!TRCanvasTest methodsFor: 'fixed shape' stamp: 'AlexandreBergel 1/5/2014 19:36'!
testRemovingFixedElement
	| c r |
	c := TRCanvas new.
	r := c buildRectangle: ((0 @ 0) extent: (150 @ 75)) color: Color red.
	r setAsFixed.

	self assert: c numberOfFixedShapes = 1.
	self assert: c numberOfShapes = 0.
	r remove.
	self assert: c numberOfShapes = 0.
	self assert: c numberOfFixedShapes = 0.! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 1/22/2014 14:27'!
testRemovingLine

	| line |
	line := TRLineShape new from: 50 @ 50 to: 100 @ 80.
	self deny: line isInACanvas.
	canvas addShape: line.
	self assert: line isInACanvas.
	
	line remove.
	self deny: line isInACanvas.
	
	line remove.
	self deny: line isInACanvas.
! !

!TRCanvasTest methodsFor: 'resizing' stamp: 'JurajKubelka 1/27/2014 10:22'!
testResizing

	| t |
	t := false.
	canvas when: TRResizeCanvasEvent do: [ :evt | t := true ].
	canvas extent:  12 @ 12.
	self assert: t! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/14/2013 18:51'!
testShape
	| shape |
	shape := canvas buildRectangle: (5 @ 5 extent: 20 @ 20) color: Color gray.
	self assert: shape notNil.
! !

!TRCanvasTest methodsFor: 'events' stamp: 'JurajKubelka 1/27/2014 10:22'!
testShapeAndEvent
	| shape t |
	shape := TRBoxShape new.
	t := false.
	shape when: TRMouseEnter do: [ :event | t := true ].
	self deny: t.
	shape announce: TRMouseEnter new.
	self assert: t! !

!TRCanvasTest methodsFor: 'events' stamp: 'JurajKubelka 1/27/2014 10:22'!
testShapeAndSharedAnnouncer
	| shape1 shape2 t |
	shape1 := TRBoxShape new.
	shape2 := TRLineShape new.
	shape1 announcer: shape2 announcer.
	
	t := 0.
	shape1 when: TRMouseLeave do: [ :event | t := t + 1 ].
	shape2 when: TRMouseEnter do: [ :event | t := t + 10 ].

	shape1 announce: TRMouseEnter new.
	shape2 announce: TRMouseLeave new.
	self assert: t = 11! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/14/2013 18:52'!
testShapes

	canvas buildRectangle: (5 @ 5 extent: 20 @ 20) color: Color gray.
	canvas buildRectangle: (5 @ 5 extent: 20 @ 20) color: Color gray.
	self assert: canvas shapes size = 2! !

!TRCanvasTest methodsFor: 'tests' stamp: 'AlexandreBergel 12/29/2013 10:04'!
testShapingElement

	| circle |
	circle := canvas buildCircleFrom: 20 @ 20 to: 100 @ 80 color: Color red.

	self assert: circle rectangle = ((20 @ 20) corner: (100 @ 80)).
	
	circle translateBy: -20 @ -20.
	self assert: circle rectangle = ((0 @ 0) corner: (80 @ 60)).! !

!TRCanvasTest methodsFor: 'should be drawn' stamp: 'AlexandreBergel 1/1/2014 15:56'!
testShouldBeDrawn

	self assert: (TRBoxShape new from: 0 @ 0 to: 10 @ 20) shouldBeDrawn.
	self deny: (TRBoxShape new from: 200 @ 0 to: 10 @ 0) shouldBeDrawn.
	self deny: (TRBoxShape new from: 200 @ 0 to: 200 @ 0) shouldBeDrawn.
	self deny: (TREllipseShape new from: 0 @ 0 to: 0 @ 0) shouldBeDrawn.
	self assert: (TREllipseShape new from: 10 @ 0 to: 50 @ 50) shouldBeDrawn! !

!TRCanvasTest methodsFor: 'label' stamp: 'AlexandreBergel 1/1/2014 18:21'!
testSizeAndLabels

	| c l1 l2 |
	c := TRCanvas new.
	l1 := TRLabelShape new fontSize: 10.
	l2 := TRLabelShape new fontSize: 15; position: 20 @ 20.
	c addShape: l1.
	c addShape: l2.
	
	self assert: l1 font ~~ l2 font
	! !

TRTest subclass: #TRConstraintTest
	instanceVariableNames: 'element1 element2 view canvas shape1 shape2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRConstraintTest commentStamp: '' prior: 0!
A ROConstraintTest is xxxxxxxxx.

Instance Variables
	element1:		<Object>
	element2:		<Object>
	view:		<Object>

element1
	- xxxxx

element2
	- xxxxx

view
	- xxxxx
!

!TRConstraintTest methodsFor: 'running' stamp: 'AlexandreBergel 1/8/2014 17:43'!
setUp
	super setUp.
	canvas := TRCanvas new.
	shape1 := TRBoxShape new size: 50.
	shape2 := TRBoxShape new size: 10.
	canvas addShape: shape1.
	canvas addShape: shape2.! !

!TRConstraintTest methodsFor: 'sticking' stamp: 'AlexandreBergel 1/8/2014 17:46'!
testStickAbove

	TRConstraint stick: shape2 above: shape1.
	self assert: shape1 center = (0 @ 0).
	self assert: shape2 center = (0 @ -30).
	
	shape1 translateBy: 10 @ 10.
	self assert: shape1 center = (10 @ 10).
	self assert: shape2 center = (10 @ -20).
! !

!TRConstraintTest methodsFor: 'sticking' stamp: 'AlexandreBergel 1/8/2014 17:58'!
testStickBelow

	TRConstraint stick: shape2 below: shape1.
	self assert: shape1 center = (0 @ 0).
	self assert: shape2 center = (0 @ 30).

	shape1 translateBy: 10 @ 10.
	self assert: shape1 center = (10 @ 10).
	self assert: shape2 center = (10 @ 40).
! !

!TRConstraintTest methodsFor: 'sticking' stamp: 'AlexandreBergel 1/8/2014 18:02'!
testStickToTheCenter

	TRConstraint 
		stick: shape1 onTheCenterOf: shape2.
		
	self assert: shape1 center = (0 @ 0).
	self assert: shape2 center = (0 @ 0).

	shape2 translateBy: 10 @ 15.
	self assert: shape1 center = (10 @ 15).
	self assert: shape2 center = (10 @ 15).
! !

TRTest subclass: #TRShapeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRShapeTest methodsFor: 'position' stamp: 'AlexandreBergel 1/28/2014 17:36'!
testPosition

	| box |
	box := TRBoxShape new.
	box translateTo: 10 @ 20.
	self assert: box position = (10 @ 20)! !

!TRShapeTest methodsFor: 'position' stamp: 'AlexandreBergel 1/28/2014 17:39'!
testPositionDefault

	self assert: TRBoxShape new position = (0 @ 0)! !

!TRShapeTest methodsFor: 'position' stamp: 'AlexandreBergel 1/28/2014 17:37'!
testPositionWithFloats

	| box |
	box := TRBoxShape new.
	box translateTo: 1.1234 @ 1.1234.
	self assert: box position = (1.1234 @ 1.1234)! !

Object subclass: #TRObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRObject subclass: #TRCamera
	instanceVariableNames: 'position canvas scale'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRCamera class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 12:54'!
forCanvas: aTrachelCanvas

	^ super new canvas: aTrachelCanvas! !

!TRCamera class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 12:54'!
new
	self error: 'Instance of this class have to be created with #forCanvas:'! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 12:56'!
canvas
	"Return the trachel canvas I am observing"
	^ canvas! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 12:57'!
canvas: aTrachelCanvas
	"Set the canvas to observe"
	canvas := aTrachelCanvas! !

!TRCamera methodsFor: 'utility' stamp: 'AlexandreBergel 12/29/2013 13:05'!
encompassingRectangle

	"Maybe a method encompassingRectangle has to be defined in TRCanvas?"
	^ self encompassingRectangleOf: canvas shapes! !

!TRCamera methodsFor: 'utility' stamp: 'AlexandreBergel 12/29/2013 13:03'!
encompassingRectangleOf: shapes
	| encompassingRectangle |
	encompassingRectangle := shapes anyOne encompassingRectangle.
	shapes do: [ :e | encompassingRectangle := encompassingRectangle merge: e encompassingRectangle ].
	^ encompassingRectangle ! !

!TRCamera methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 13:07'!
focusOnCenter
	"Do nothing if no shape are present"
	canvas numberOfShapes = 0 ifTrue: [ ^ self ]. 
	self translateTo: self encompassingRectangle center! !

!TRCamera methodsFor: 'convertion' stamp: 'AlexandreBergel 12/29/2013 13:28'!
fromPixelToSpace: aPoint
	| tr |
	tr := ((canvas extent / 2) - self position) negated.
	^ AthensAffineTransform new translation: tr; transform: aPoint! !

!TRCamera methodsFor: 'convertion' stamp: 'AlexandreBergel 12/29/2013 13:22'!
fromSpaceToPixel: aPoint
	| tr |
	tr := (canvas extent / 2) - self position.
	^ AthensAffineTransform new translation: tr; transform: aPoint! !

!TRCamera methodsFor: 'initialization' stamp: 'AlexandreBergel 1/8/2014 18:17'!
initialize
	super initialize.
	position := 0 @ 0.
	scale := 1.0! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 11:37'!
position
	^ position! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 18:20'!
scale
	^ scale! !

!TRCamera methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 18:20'!
scale: aNewScale
	scale := aNewScale! !

!TRCamera methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 11:38'!
translateBy: aPoint
	position := position + aPoint! !

!TRCamera methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 11:38'!
translateTo: newPosition
	position := newPosition! !

TRObject subclass: #TRCanvas
	instanceVariableNames: 'canvas shapes fixedShapes extent animations morph camera announcer color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 1/28/2014 15:29'!
addAnimation: anAnimation
	animations add: anAnimation.
	anAnimation addedIn: self.
	anAnimation start.
	self signalUpdate! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/31/2013 09:33'!
addFixedShape: shape
	fixedShapes add: shape.
	shape canvas: self! !

!TRCanvas methodsFor: 'menu' stamp: 'JurajKubelka 1/27/2014 10:22'!
addMenu: aTitle callback: aBlock
	| lbl index |
	index := fixedShapes inject: 0 into: [ :s :e | s max: (e encompassingRectangle topRight x + 10) ].
	lbl := TRLabelShape new text: aTitle; leftPosition: index @ 20.
	lbl color: Color black.
	lbl when: TRMouseClick do: [ :event | aBlock value ].
	lbl when: TRMouseEnter do: [ :event | lbl color: Color blue. self signalUpdate ].
	lbl when: TRMouseLeave do: [ :event | lbl color: Color black. self signalUpdate ].
	self addFixedShape: lbl! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 1/13/2014 18:53'!
addShape: shape
	shape addedIn: self! !

!TRCanvas methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:49'!
announce: anEvent
	self announcer announce: anEvent! !

!TRCanvas methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:54'!
announcer
	announcer isNil ifTrue: [ announcer := Announcer new ].
	^ announcer! !

!TRCanvas methodsFor: 'events' stamp: 'AlexandreBergel 12/31/2013 08:54'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/25/2013 09:26'!
athensCanvas
	^ canvas! !

!TRCanvas methodsFor: 'building' stamp: 'SamirSaleh 1/21/2014 10:43'!
buildArcAlphaAngle: stAngle betaAngle: ndAngle innerRadius: iR externalRadius: eR
	| shape |
	shape := TRArcShape new.
	shape alphaAngle: stAngle betaAngle: ndAngle innerRadius: iR externalRadius: eR.
	self addShape: shape.
	^ shape! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/24/2013 18:44'!
buildCircleFrom: aPoint to: anotherPoint color: aColor
	| shape |
	shape := TREllipseShape new.
	shape from: aPoint to: anotherPoint color: aColor.
	self addShape: shape.
	^ shape! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/24/2013 18:35'!
buildLineFrom: aPoint to: anotherPoint color: aColor
	|  shape |
	shape := TRLineShape new.
	shape from: aPoint to: anotherPoint color: aColor.
	self addShape: shape.
	^ shape! !

!TRCanvas methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 1/6/2014 13:17'!
buildMorph
	morph := TRMorph new
		canvas: self;
		extent: canvas extent.
	^ morph! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 12/24/2013 18:34'!
buildRectangle: aRectangle color: aColor
	| shape |
	shape := TRBoxShape new.
	shape fromRectangle: aRectangle color: aColor.
	self addShape: shape.
	^ shape! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 11:47'!
camera
	^ camera! !

!TRCanvas methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/8/2014 19:47'!
color
	^ color! !

!TRCanvas methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/8/2014 19:52'!
color: aColor
	color := aColor.
	self signalUpdate! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 13:36'!
extent
	"size of the canvas"
	^ extent! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 17:15'!
extent: aPoint
	"Set of the size of the canvas. Useful, for example, when the window is resized"
	extent := aPoint.
	self announce: TRResizeCanvasEvent! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/31/2013 10:20'!
fixedShapes
	^ fixedShapes! !

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 12/29/2013 13:36'!
hasAnimation
	"True is at least one animation is running"
	^ animations notEmpty! !

!TRCanvas methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 1/8/2014 19:47'!
initialize
	super initialize.
	extent := 500 @ 500.
	canvas := AthensCairoSurface extent: extent.
	shapes := OrderedCollection new.
	fixedShapes := OrderedCollection new.
	animations := OrderedCollection new.
	camera := TRCamera forCanvas: self.
	color := Color lightGray.! !

!TRCanvas methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:09'!
isFixedShape: aShape
	^ fixedShapes includes: aShape! !

!TRCanvas methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:09'!
isMovableShape: aShape
	^ shapes includes: aShape! !

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 12/29/2013 13:35'!
numberOfAnimations
	"Return the number of animations contained in the canvas"
	^ animations size! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 12/31/2013 09:19'!
numberOfFixedShapes
	"Return the number of displayed shapes"
	^ fixedShapes size! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 1/5/2014 19:36'!
numberOfShapes
	"Return the number of displayed nonfixed shapes"
	^ shapes size ! !

!TRCanvas methodsFor: 'instance creation' stamp: 'AlexandreBergel 1/17/2014 09:39'!
open
	^ self openInWindowSized: 500 @ 500! !

!TRCanvas methodsFor: 'instance creation' stamp: 'AlexandreBergel 1/17/2014 09:38'!
openInWindowSized: anExtentAsPoint
	| window |
	self buildMorph.
	window := morph openInWindow.
	window extent: anExtentAsPoint.
	^ window! !

!TRCanvas methodsFor: 'instance creation' stamp: 'AlexandreBergel 1/22/2014 15:40'!
openInWindowSized: anExtentAsPoint titled: aTitle
	| window |
	window := self openInWindowSized: anExtentAsPoint.
	window setLabel: aTitle.
	^ window! !

!TRCanvas methodsFor: 'instance creation' stamp: 'AlexandreBergel 1/22/2014 15:40'!
openTitled: aTitle
	^ self openInWindowSized: 500 @ 500 titled: aTitle! !

!TRCanvas methodsFor: 'animations' stamp: 'AlexandreBergel 12/23/2013 14:06'!
playAnimations
	| shouldClean |
	shouldClean := false.
	animations do: [ :animation | 
		animation refresh. 
		animation hasCompleted ifTrue: [ shouldClean := true ] ].
	
	shouldClean ifTrue: [ 
		animations := animations reject: [ :animation | animation hasCompleted ] ].! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 1/13/2014 17:51'!
privateAdd: shape
	shapes add: shape! !

!TRCanvas methodsFor: 'actions' stamp: 'AlexandreBergel 1/8/2014 15:23'!
pushBack: aShape
	shapes remove: aShape.
	shapes addFirst: aShape! !

!TRCanvas methodsFor: 'actions' stamp: 'AlexandreBergel 1/8/2014 15:22'!
pushFront: aShape
	shapes remove: aShape.
	shapes addLast: aShape! !

!TRCanvas methodsFor: 'actions' stamp: 'AlexandreBergel 12/31/2013 09:59'!
removeFixedShape: aShape
	fixedShapes := fixedShapes copyWithout: aShape! !

!TRCanvas methodsFor: 'building' stamp: 'AlexandreBergel 1/5/2014 19:35'!
removeShape: shape
	shapes remove: shape ifAbsent: [ self removeFixedShape: shape ].
	shape canvas: nil! !

!TRCanvas methodsFor: 'actions' stamp: 'AlexandreBergel 1/5/2014 19:14'!
setAsFixed: aShape

	self removeShape: aShape.
	self addFixedShape: aShape.
	
! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 1/15/2014 16:07'!
shapeForPosition: position


	fixedShapes reverseDo: [ :s | 
		(s includesPoint: position) ifTrue: [ ^ s ] ].

	shapes reverseDo: [ :s | 
		(s includesPoint: position) ifTrue: [ ^ s ] ].
	^ self! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 15:14'!
shapeForPositionInPixels: position
	"position is in pixel, it corresponds to the physical location in the window.
	e.g., 0 @ 0 is top left of the window"
	fixedShapes reverseDo: [ :s | 
		(s includesPoint: position) ifTrue: [ ^ s ] ].

	shapes reverseDo: [ :s | 
		(s includesPoint: (self camera fromPixelToSpace: position)) ifTrue: [ ^ s ] ].
	^ self! !

!TRCanvas methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 15:20'!
shapes
	"Return the list of shapes"
	^ shapes asArray! !

!TRCanvas methodsFor: 'updating' stamp: 'AlexandreBergel 12/25/2013 09:28'!
signalUpdate
	morph changed! !

!TRCanvas methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 1/8/2014 20:16'!
topLeft

	^ morph
		ifNil:[ 0@0 ]
		ifNotNil: [ morph topLeft ]! !

!TRCanvas methodsFor: 'events' stamp: 'JurajKubelka 1/27/2014 10:22'!
when: event do: aBlock
	self announcer when: event do: aBlock! !

TRObject subclass: #TRCanvasExample
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRCanvasExample methodsFor: 'arcs' stamp: 'SamirSaleh 1/21/2014 10:06'!
arc
	"
	self new arc
	"
	| c  shape|
	c := TRCanvas new.
	shape:= TRArcShape new.
	shape alphaAngle:10  betaAngle: 80  innerRadius: 30 externalRadius: 150. 
	shape color: Color blue.
	c addShape: shape.
	c open.
	
! !

!TRCanvasExample methodsFor: 'arcs' stamp: 'AlexandreBergel 1/24/2014 19:20'!
arc0
	"
	self new arc0
	"
	| c shape |
	c := TRCanvas new.
	shape:= TRArcShape new.
	shape alphaAngle:0  betaAngle: 360  innerRadius: 0 externalRadius: 100. 
	shape color: Color blue.
	c addShape: shape.
	c open.
	
! !

!TRCanvasExample methodsFor: 'arcs' stamp: 'SamirSaleh 1/21/2014 10:06'!
arc2
	"
	self new arc
	"
	| c  shape1 shape2 shape3 shape4|
	c := TRCanvas new.
	shape1:= TRArcShape new.
	shape2:= TRArcShape new.
	shape3:= TRArcShape new.
	shape4:= TRArcShape new.
	shape1 alphaAngle:10  betaAngle: 80 innerRadius: 30 externalRadius: 150. 
	shape1 color: Color blue.
	shape2 alphaAngle:100  betaAngle: 170 innerRadius: 30 externalRadius: 150. 
	shape2 color: Color green.
	shape3 alphaAngle:190  betaAngle: 260 innerRadius: 30 externalRadius: 150. 
	shape3 color: Color yellow.
	shape4 alphaAngle:280  betaAngle: 350 innerRadius: 30 externalRadius: 150. 
	shape4 color: Color red.
	c addShape: shape1.c addShape: shape2. c addShape: shape3. c addShape: shape4.
	c open.
	
! !

!TRCanvasExample methodsFor: 'arcs' stamp: 'SamirSaleh 1/21/2014 10:06'!
arc3
	"
	self new arc3
	"
	| c  shape1 shape2 shape3 shape4|
	c := TRCanvas new.
	shape1:= TRArcShape new.
	shape2:= TRArcShape new.
	shape3:= TRArcShape new.
	shape4:= TRArcShape new.
	shape1 alphaAngle:5  betaAngle: 85 innerRadius: 30 externalRadius: 150. 
	shape1 color: Color blue.
	shape2 alphaAngle:95  betaAngle: 175 innerRadius: 30 externalRadius: 150. 
	shape2 color: Color green.
	shape3 alphaAngle:185  betaAngle: 265 innerRadius: 30 externalRadius: 150. 
	shape3 color: Color yellow.
	shape4 alphaAngle:275  betaAngle: 355 innerRadius: 30 externalRadius: 150. 
	shape4 color: Color red.
	c addShape: shape1.c addShape: shape2. c addShape: shape3. c addShape: shape4.
	c open.
	
! !

!TRCanvasExample methodsFor: 'arcs' stamp: 'SamirSaleh 1/21/2014 10:06'!
arc4
	"
	self new arc4
	"
	| c  shape1 shape2 shape3 shape4|
	c := TRCanvas new.
	shape1:= TRArcShape new.
	shape2:= TRArcShape new.
	shape3:= TRArcShape new.
	shape4:= TRArcShape new.
	shape1 alphaAngle:0  betaAngle: 90 innerRadius: 130 externalRadius: 150. 
	shape1 color: Color blue.
	shape2 alphaAngle:90  betaAngle: 180 innerRadius: 130 externalRadius: 150. 
	shape2 color: Color green.
	shape3 alphaAngle:180  betaAngle: 270 innerRadius: 130 externalRadius: 150. 
	shape3 color: Color yellow.
	shape4 alphaAngle:270  betaAngle: 360 innerRadius: 130 externalRadius: 150. 
	shape4 color: Color red.
	c addShape: shape1.c addShape: shape2. c addShape: shape3. c addShape: shape4.
	c open.
	
! !

!TRCanvasExample methodsFor: 'arcs' stamp: 'SamirSaleh 1/21/2014 10:06'!
arc5
	"
	self new arc5
	"
	| c  shape1 shape2 shape3 shape4|
	c := TRCanvas new.
	shape1:= TRArcShape new.
	shape2:= TRArcShape new.
	shape3:= TRArcShape new.
	shape4:= TRArcShape new.
	shape1 alphaAngle:0  betaAngle: 90 innerRadius: 0 externalRadius: 150. 
	shape1 color: Color blue.
	shape2 alphaAngle:90  betaAngle: 180 innerRadius: 0 externalRadius: 150. 
	shape2 color: Color green.
	shape3 alphaAngle:180  betaAngle: 270 innerRadius: 0 externalRadius: 150. 
	shape3 color: Color yellow.
	shape4 alphaAngle:270  betaAngle: 360 innerRadius: 0 externalRadius: 150. 
	shape4 color: Color red.
	c addShape: shape1.c addShape: shape2. c addShape: shape3. c addShape: shape4.
	c open.
	
! !

!TRCanvasExample methodsFor: 'arcs' stamp: 'AlexandreBergel 1/24/2014 18:14'!
arc6
	"
	self new arc6
	"
	| c  shape shape2|
	c := TRCanvas new.
	shape:= TRArcShape new.
	shape alphaAngle:10  betaAngle: 80  innerRadius: 30 externalRadius: 150. 
	shape color: Color blue.
	shape2:=TRBoxShape new.
	shape2 rectangle: (shape topLeft corner: shape bottomRight).
	shape2 color: Color red.
	c addShape: shape.
	c addShape: shape2.
	c open.
	
! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 1/11/2014 21:41'!
bezier

	| t |
	t := TRCanvas new.
	t addShape: (TRLineShape  new from: 50 @ 50 to: 130 @ 20).
	t addShape: (TRBezierShape new 
		points: {50 @ 50 . 150 @ 20 . 120 @ 130 . 50 @ 150 . 80 @ 80}; 
			strokePaint: Color purple).
	t open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 1/22/2014 15:41'!
circle
	"
	self new circle
	"
	| c  |
	c := TRCanvas new.
	c buildRectangle: (50 @ 50 corner: 80 @ 80) color: (Color blue alpha: 0.2).

	100 to: 200 by: 20 do: [ :i |
		 c buildCircleFrom: 50 @ 50 to: i @ 100 color: (Color red alpha: 0.1).
	].
	c openTitled: 'Example with transparent circles'.
	
! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/29/2013 11:58'!
circle2
	"
	self new circle2
	"

	| c circle line1 line2 |
	c := TRCanvas new.
	circle := TREllipseShape new from: -50 @ -50 to: 50 @ 50.
	circle color: (Color purple alpha: 0.3).
	line1 := TRLineShape new from: -50 @ 0 to: 50 @ 0.
	line2 := TRLineShape new from: 0 @ -50 to: 0 @ 50.
	line1 color: (Color blue alpha: 0.5).
	line2 color: (Color blue alpha: 0.5).
	
	c addShape: circle.
	c addShape: line1.
	c addShape: line2.
	c open! !

!TRCanvasExample methodsFor: 'rotating labels' stamp: 'JurajKubelka 1/20/2014 15:54'!
fiveLabelFlowers
	"
	self new fiveLabelFlowers.
	"
	| t label box |
	t := TRCanvas new.
	t color: Color white.

	{0 @ 0. 150 @ 150. -150 @ -150. 150 @ -150. -150 @ 150} do: [ :position |
		1 to: 180 by: 25 do: [ :angle |
			box := TRBoxShape new.
			box from: position -1 to: position + 1.
			box color: (Color blue alpha: 0.5).
			t addShape: box.
			
			label := TRRotatedLabelShape new.
			label text: 'Hello World Bonjour Monde'.
			label color: Color black.
			label position: position.
			label angleInDegree: angle.
			t addShape: label ] ].
	
	t open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'JurajKubelka 1/27/2014 10:22'!
fixedRec
	"
	self new fixedRec
	"

	| c r1 r2 colors |
	c := TRCanvas new.
	r1 := c buildRectangle: ((0 @ 0) extent: (150 @ 75)) color: Color red.
	r2 := c buildRectangle: ((0 @ 0) extent: (130 @ 50)) color: Color lightGreen.
	c buildLineFrom: 50 @ 50 to: 80 @ 80 color: Color lightGreen.
	
	colors := Array with: Color red with: Color blue with: Color white.
	r1 when: TRMouseClick do: [ :event | event shape color: colors atRandom. c signalUpdate ].
	r2 when: TRMouseClick do: [ :event | r2 color: colors atRandom. c signalUpdate ].	
		
	r2 setAsFixed.
	
	c when: TRMouseDragging do: [ :event | c camera translateBy: event step negated. c signalUpdate ].
	c open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 1/1/2014 18:22'!
label
	"
	self new label
	"
	| c |
	c := TRCanvas new.
	c addShape: (TRLabelShape new fontSize: 5).
	c addShape: (TRLabelShape new fontSize: 25; position: 20 @ 20).
	c open! !

!TRCanvasExample methodsFor: 'menu' stamp: 'AlexandreBergel 1/1/2014 14:01'!
menu1
	"
	self new menu1
	"
	| canvas |
	canvas := TRCanvas new.
	canvas addMenu: '+ box' callback: [ canvas addShape: (TRBoxShape new color: (Color purple alpha: 0.1); from: (400 atRandom @ 400 atRandom) extent: (100 atRandom @ 100 atRandom)). canvas signalUpdate ].

	canvas addMenu: '+ circle' callback: [ canvas addShape: (TREllipseShape new color: (Color blue alpha: 0.1); from: (400 atRandom @ 400 atRandom) extent: (100 atRandom @ 100 atRandom)). canvas signalUpdate ].	
	canvas camera translateTo: 250 @ 250.
	canvas open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/14/2013 17:49'!
rec1
	"
	self new rec1
	"

	| c |
	c := TRCanvas new.
	c buildRectangle: ((50 @ 50) extent: (150 @ 75)) color: Color red.
	c buildRectangle: ((80 @ 80) extent: (150 @ 75)) color: Color lightGreen.
	c buildLineFrom: 50 @ 50 to: 80 @ 80 color: Color lightGreen.
	c open! !

!TRCanvasExample methodsFor: 'examples' stamp: 'AlexandreBergel 12/31/2013 11:20'!
text
	"
	self new text
	"
	| c label rec |
	c := TRCanvas new.
	label := TRLabelShape new.
	rec := TRBoxShape new color: Color white.
	rec fromRectangle: label encompassingRectangle.
	c addShape: rec.
	c addShape: label.
	c open! !

TRObject subclass: #TRConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRConstraint commentStamp: '<historical>' prior: 0!
A ROConstraint is a utility class that contains methods to position elements against others.
!

!TRConstraint class methodsFor: 'util' stamp: 'AlexandreBergel 1/8/2014 17:41'!
move: aShape above: anotherShape
	^ aShape translateTo: anotherShape center - (0 @ ((anotherShape height + aShape height) / 2)) ! !

!TRConstraint class methodsFor: 'util' stamp: 'AlexandreBergel 1/8/2014 17:56'!
move: aShape below: anotherShape
	^ aShape translateTo: anotherShape center + (0 @ ((anotherShape height + aShape height) / 2)) ! !

!TRConstraint class methodsFor: 'util' stamp: 'RicardoJacas 1/22/2014 10:19'!
move: aShape between: s1 and: s2

	^ aShape translateTo: ((s1 position + s2 position) / 2)! !

!TRConstraint class methodsFor: 'util' stamp: 'RicardoJacas 1/22/2014 10:12'!
move: aShape onTheCenterOf: anotherShape
	^ aShape translateTo: anotherShape center
! !

!TRConstraint class methodsFor: 'util' stamp: 'RicardoJacas 1/22/2014 15:41'!
move: aShape onTheLeftOf: anotherShape
	^ aShape translateTo: anotherShape center - ((((anotherShape width/2) rounded) + (aShape width/2 rounded)) @ 0)! !

!TRConstraint class methodsFor: 'util' stamp: 'RicardoJacas 1/22/2014 15:41'!
move: aShape onTheRightOf: anotherShape
	^ aShape translateTo: anotherShape center + ((((anotherShape width/2) rounded) + (aShape width/2 rounded)) @ 0)! !

!TRConstraint class methodsFor: 'util' stamp: 'AlexandreBergel 1/17/2014 10:55'!
moveAtTheBottomOfTheWindow: trachelShape

	| windowSize |
	windowSize := trachelShape canvas extent.
	trachelShape translateTo: ((trachelShape width / 2) @ (windowSize y - (trachelShape height / 2)))! !

!TRConstraint class methodsFor: 'camera' stamp: 'AlexandreBergel 1/8/2014 17:33'!
moveCameraToTheRightIn: trachelCanvas
	^ trachelCanvas camera translateTo: (trachelCanvas extent x / -2) @ trachelCanvas camera position y! !

!TRConstraint class methodsFor: 'public' stamp: 'AlexandreBergel 1/8/2014 17:55'!
stick: aShape above: anotherShape
	self move: aShape above: anotherShape. 
	anotherShape addCallback: 
		(TRTranslationCallback block: [ :shape :step | self move: aShape above: anotherShape ]) 
	
	! !

!TRConstraint class methodsFor: 'public' stamp: 'AlexandreBergel 1/8/2014 17:58'!
stick: aShape below: anotherShape
	self move: aShape below: anotherShape. 
	anotherShape addCallback: 
		(TRTranslationCallback block: [ :shape :step | self move: aShape below: anotherShape ]) 
	
	! !

!TRConstraint class methodsFor: 'public' stamp: 'AlexandreBergel 1/8/2014 18:01'!
stick: aShape onTheCenterOf: anotherShape
	self move: aShape onTheCenterOf: anotherShape. 
	anotherShape addCallback: 
		(TRTranslationCallback block: [ :shape :step | self move: aShape onTheCenterOf: anotherShape ]) 
	! !

!TRConstraint class methodsFor: 'public' stamp: 'RicardoJacas 1/22/2014 10:22'!
stick: aShape onTheLeftOf: anotherShape
	self move: aShape onTheLeftOf: anotherShape. 
	anotherShape addCallback: 
		(TRTranslationCallback block: [ :shape :step | self move: aShape onTheLeftOf: anotherShape ]) 
	! !

!TRConstraint class methodsFor: 'public' stamp: 'RicardoJacas 1/22/2014 10:23'!
stick: aShape onTheRightOf: anotherShape
	self move: aShape onTheRightOf: anotherShape. 
	anotherShape addCallback: 
		(TRTranslationCallback block: [ :shape :step | self move: aShape onTheRightOf: anotherShape ]) 
	! !

!TRConstraint class methodsFor: 'public' stamp: 'JurajKubelka 1/27/2014 10:22'!
stickAtTheBottomOfTheWindow: trachelShape
	trachelShape canvas
		when: TRResizeCanvasEvent
		do: [ :event | self moveAtTheBottomOfTheWindow: trachelShape ]! !

!TRConstraint class methodsFor: 'camera' stamp: 'JurajKubelka 1/27/2014 10:22'!
stickCameraToTheRightIn: trachelCanvas
	trachelCanvas
		when: TRResizeCanvasEvent
		do: [ :event | self moveCameraToTheRightIn: trachelCanvas ]! !

TRObject subclass: #TRFocusing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRFocusing subclass: #TRFocusCenter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRFocusCenter methodsFor: 'hooks' stamp: 'AlexandreBergel 12/29/2013 12:06'!
on: canvas
	| encompassingRectangle  |
	encompassingRectangle := self encompassingRectangleOf: canvas shapes.
	canvas camera translateTo: encompassingRectangle center.! !

TRFocusing subclass: #TRFocusTopLeft
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRFocusTopLeft methodsFor: 'hooks' stamp: 'AlexandreBergel 12/29/2013 12:07'!
on: canvas
	| encompassingRectangle |
	encompassingRectangle := self encompassingRectangleOf: canvas shapes.
	canvas camera translateTo: encompassingRectangle topLeft! !

TRFocusing subclass: #TRFocusTopRight
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRFocusTopRight methodsFor: 'hooks' stamp: 'AlexandreBergel 12/29/2013 12:07'!
on: canvas
	| encompassingRectangle |
	encompassingRectangle := self encompassingRectangleOf: canvas shapes. 
	canvas camera translateTo: encompassingRectangle topRight! !

!TRFocusing class methodsFor: 'public' stamp: 'AlexandreBergel 12/28/2013 15:14'!
on: canvas
	^ self new on: canvas! !

!TRFocusing methodsFor: 'utility' stamp: 'AlexandreBergel 12/28/2013 15:23'!
encompassingRectangleOf: shapes
	| encompassingRectangle |
	encompassingRectangle := shapes anyOne encompassingRectangle.
	shapes do: [ :e | encompassingRectangle := encompassingRectangle merge: e encompassingRectangle ].
	^ encompassingRectangle ! !

!TRFocusing methodsFor: 'hooks' stamp: 'AlexandreBergel 12/28/2013 15:14'!
on: canvas
	self subclassResponsibility! !

TRObject subclass: #TRShape
	instanceVariableNames: 'canvas color path strokePaint announcer element callbacks position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRShape subclass: #TRAbstractBoxShape
	instanceVariableNames: 'rectangle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRAbstractBoxShape class
	instanceVariableNames: 'nullRectangle'!

TRAbstractBoxShape class
	instanceVariableNames: 'nullRectangle'!

!TRAbstractBoxShape class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 11:15'!
nullRectangle
	nullRectangle ifNotNil: [ ^ nullRectangle ].
	nullRectangle := (0 @ 0) extent: (0 @ 0).
	^ nullRectangle! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/28/2014 17:43'!
center
	^ self position! !

!TRAbstractBoxShape methodsFor: 'hooks' stamp: 'AlexandreBergel 1/18/2014 19:02'!
computePath
	canvas ifNil: [ ^ self ].
	path := self athensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveTo: rectangle topLeft;
				lineTo: rectangle topRight;
				lineTo: rectangle bottomRight;
				lineTo: rectangle bottomLeft;
				lineTo: rectangle topLeft.
				"moveTo: -1 @ -1;
				lineTo: 1 @ -1;
				lineTo: 1 @ 1;
				lineTo: -1 @ 1;
				lineTo: -1 @ -1" ]! !

!TRAbstractBoxShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/18/2014 19:02'!
drawOn: athensCanvas
	self shouldBeDrawn
		ifFalse: [ ^ self ].
	athensCanvas pathTransform
		restoreAfter: [ 
			"athensCanvas pathTransform
				translateBy: rectangle center;
				scaleBy: rectangle extent / 2."
			athensCanvas
				setPaint: color;
				drawShape: self path.
			self drawStrokeIfNecessaryOn: athensCanvas ]! !

!TRAbstractBoxShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/18/2014 22:39'!
drawStrokeIfNecessaryOn: athensCanvas
	| scale |
	scale := self canvas camera scale.
	^ strokePaint
		ifNotNil: [ 
			"4 / (rectangle extent x + 10)"
			(athensCanvas setStrokePaint: strokePaint) width: (1.5 / (scale )) asInteger.
			athensCanvas drawShape: path ]! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/28/2013 15:21'!
encompassingRectangle
	^ self rectangle! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 15:59'!
extent
	^ self rectangle extent! !

!TRAbstractBoxShape methodsFor: 'geometry' stamp: 'AlexandreBergel 12/24/2013 16:59'!
extent: anExtent
	| center |
	center := rectangle center.
	rectangle := (center - (anExtent / 2)) extent: anExtent.
	self resetPath! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/31/2013 09:42'!
from: aPoint extent: anotherPoint 
	
	self fromRectangle: (aPoint corner: (aPoint + anotherPoint))! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/16/2013 16:52'!
from: aPoint to: anotherPoint 
	
	self fromRectangle: (aPoint corner: anotherPoint)! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/16/2013 16:52'!
from: aPoint to: anotherPoint color: aColor
	self from: aPoint to: anotherPoint.
	self color: aColor.! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 1/28/2014 17:41'!
fromRectangle: r
	| oldTopLeft |
	oldTopLeft := rectangle topLeft.
	rectangle := r.
	position := rectangle center.
	self triggerCallbacksForStep: (r topLeft - oldTopLeft).
	self resetPath! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 12/16/2013 16:52'!
fromRectangle: r color: aColor
	self fromRectangle: r.
	self color: aColor! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 23:41'!
height
	^ rectangle height! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/28/2014 17:43'!
height: newHeight
	rectangle := (position - (0 @ (newHeight / 2))) extent: (self extent x @ newHeight)! !

!TRAbstractBoxShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 11:02'!
includesPoint: aPoint
	^ rectangle containsPoint: aPoint! !

!TRAbstractBoxShape methodsFor: 'initialization' stamp: 'AlexandreBergel 12/29/2013 11:16'!
initialize
	super initialize.
	rectangle := self class nullRectangle! !

!TRAbstractBoxShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 14:29'!
leftPosition: aPosition
	self position: (aPosition - ((self width / -2) @ 0) ) asIntegerPoint! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 15:48'!
rectangle
	^ rectangle! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 18:46'!
rectangle: r
	self fromRectangle: r! !

!TRAbstractBoxShape methodsFor: 'testing' stamp: 'AlexandreBergel 1/1/2014 15:54'!
shouldBeDrawn
	^ rectangle width > 0 and: [ rectangle height > 0 ]! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/8/2014 15:13'!
size: v
	^ self extent: v @ v! !

!TRAbstractBoxShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/28/2014 17:55'!
translateBy: aPoint
	super translateBy: aPoint.
	rectangle := (rectangle topLeft + aPoint) extent: rectangle extent.
	self triggerCallbacksForStep: aPoint! !

!TRAbstractBoxShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/28/2014 17:42'!
translateTo: aPoint
	| oldTopLeft |
	super translateTo: aPoint.
	oldTopLeft := rectangle topLeft.
	rectangle := (aPoint - (self extent / 2)) extent: self extent.
	self triggerCallbacksForStep: (rectangle topLeft - oldTopLeft)! !

!TRAbstractBoxShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:55'!
triggerCallbacksForStep: aStep
	"aStep is a point that represents a translation step"
	callbacks ifNil: [ ^ self ].
	callbacks do: [ :c | c block value: self value: aStep ]! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/23/2013 23:40'!
width
	^ rectangle width! !

!TRAbstractBoxShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/28/2014 17:43'!
width: newWidth
	rectangle := (position - ((newWidth / 2) @ 0)) extent: (newWidth @ self extent y)! !

TRAbstractBoxShape subclass: #TRBoxShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRAbstractBoxShape subclass: #TREllipseShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TREllipseShape methodsFor: 'hooks' stamp: 'AlexandreBergel 12/25/2013 09:27'!
computePath
	canvas ifNil: [ ^ self ].
	path := self athensCanvas
				createPath: [ :builder | 
					builder
						absolute;
						moveTo: 0 @ 1;
						ccwArcTo: 1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0.0 @ -1.0 angle: 90 degreesToRadians;
						ccwArcTo: -1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0 @ 1.0 angle: 90 degreesToRadians ]
"	path := canvas
		cacheAt: self
		ifAbsentPut: [ 
			canvas
				createPath: [ :builder | 
					builder
						absolute;
						moveTo: 0 @ 1;
						ccwArcTo: 1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0.0 @ -1.0 angle: 90 degreesToRadians;
						ccwArcTo: -1.0 @ 0.0 angle: 90 degreesToRadians;
						ccwArcTo: 0 @ 1.0 angle: 90 degreesToRadians ] ]"! !

!TREllipseShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/3/2014 12:12'!
drawOn: athensCanvas
	self shouldBeDrawn ifFalse: [ ^ self ].
	athensCanvas pathTransform restoreAfter: [
		athensCanvas pathTransform 
			translateBy: rectangle center;
			scaleBy: rectangle extent / 2.

		athensCanvas
			setPaint: color;
			drawShape: self path.

		self drawStrokeIfNecessaryOn: athensCanvas
		]! !

!TREllipseShape methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 09:59'!
from: aPoint to: anotherPoint color: aColor
	rectangle := aPoint corner: anotherPoint.
	self strokePaint: aColor.
	self color: aColor! !

!TREllipseShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/24/2013 18:45'!
includesPoint: aPoint
	"We could aternatively translate rectangle by position"
	^ rectangle containsPoint: (aPoint)! !

TRShape subclass: #TRArcShape
	instanceVariableNames: 'alphaAngle betaAngle innerRadius externalRadius center origin height width topLeft bottomRight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRArcShape commentStamp: 'SamirSaleh 1/27/2014 09:31' prior: 0!
A TRArc is an class that makes all the drawing of arcs!

!TRArcShape methodsFor: 'initialization' stamp: 'AlexandreBergel 1/24/2014 18:13'!
alphaAngle: stAngle betaAngle: ndAngle innerRadius: iR externalRadius: eR
	alphaAngle := stAngle.
	betaAngle := ndAngle.
	innerRadius := iR.
	externalRadius := eR 
	! !

!TRArcShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/24/2014 18:13'!
bottomRight
	self computeBottomRight.
	^ bottomRight ! !

!TRArcShape methodsFor: 'accessing' stamp: 'SamirSaleh 1/21/2014 11:50'!
center
	^center! !

!TRArcShape methodsFor: 'accessing' stamp: 'SamirSaleh 1/21/2014 11:51'!
center: aPoint
	center:=aPoint! !

!TRArcShape methodsFor: 'computing' stamp: 'SamirSaleh 1/24/2014 15:56'!
computeBottomRight
	|aA bA iR eR p1 p2 p3 p4 pR pQ1 pB pQ2 pL pQ3 pT pQ4 pCollection|
	pCollection:= OrderedCollection new.
	aA:=(alphaAngle \\ 360) degreesToRadians .
	bA:=(betaAngle \\ 360) degreesToRadians .
	iR:= innerRadius.
	eR:= externalRadius.
	p1:= (iR*aA cos)@(iR*aA sin) .
	p2:= (iR*bA cos)@(iR*bA sin) .
	p3:= (eR*aA cos)@(eR*aA sin).
	p4:=(eR*bA cos)@(eR*bA sin) .
	pR:=(eR*0 cos) @ (eR*0 sin).
	pQ1:=(eR*(Float pi /4) cos)@(eR*(Float pi /4) sin).
	pB:=(eR*(Float pi /2) cos)@(eR*(Float pi /2) sin).
	pQ2:=(eR*(Float pi * (3/4)) cos)@(eR*(Float pi * (3/4)) sin).
	pL:=(eR*Float pi cos)@(eR*Float pi sin).
	pQ3:=(eR*(Float pi * (5/4)) cos)@(eR*(Float pi * (5/4)) sin).
	pT:=(eR*(Float pi * (3/2)) cos)@(eR*(Float pi * (3/2)) sin).
	pQ4:=(eR*(Float pi * (7/4)) cos)@(eR*(Float pi * (7/4)) sin).
	pCollection add:p1; add: p2; add: p3; add: p4.
	(self includesPoint: pQ1)ifTrue:[pCollection add: pQ1].
	(self includesPoint: pQ2)ifTrue:[pCollection add: pQ2].
	(self includesPoint: pQ3)ifTrue:[pCollection add: pQ3].
	(self includesPoint: pQ4)ifTrue:[pCollection add: pQ4].
	(self includesPoint: pL)ifTrue:[pCollection add: pL].
	(self includesPoint: pR)ifTrue:[pCollection add: pR].
	(self includesPoint: pB)ifTrue:[pCollection add: pB].
	(self includesPoint: pT)ifTrue:[pCollection add: pT].
	bottomRight:=pCollection max x @ pCollection min y! !

!TRArcShape methodsFor: 'computing' stamp: 'SamirSaleh 1/27/2014 11:00'!
computeHeight
	height:=(self topLeft y - self bottomRight y) abs! !

!TRArcShape methodsFor: 'hooks' stamp: 'SamirSaleh 1/21/2014 13:12'!
computePath
	|aAngle bAngle|
	aAngle:=alphaAngle degreesToRadians .
	bAngle:=betaAngle  degreesToRadians .
	canvas ifNil: [ ^ self ].
	path := self athensCanvas
				createPath: [ :builder | 
					builder
						absolute;
						moveTo: ((innerRadius * (aAngle cos)) @ (innerRadius * (aAngle sin)));
						arcTo: ((innerRadius * (bAngle cos)) @ (innerRadius * (bAngle sin))) angle: bAngle - aAngle cw: true;
						ccwArcTo: ((externalRadius * (bAngle cos)) @ (externalRadius * (bAngle sin))) angle: 0 degreesToRadians;
						ccwArcTo: ((externalRadius * (aAngle cos)) @ (externalRadius * (aAngle sin)))angle: bAngle - aAngle;
						ccwArcTo: ((innerRadius * (aAngle cos)) @ (innerRadius * (aAngle sin)))angle: 0 degreesToRadians ]! !

!TRArcShape methodsFor: 'computing' stamp: 'SamirSaleh 1/24/2014 15:54'!
computeTopLeft
	|aA bA iR eR p1 p2 p3 p4 pR pQ1 pB pQ2 pL pQ3 pT pQ4 pCollection|
	pCollection:= OrderedCollection new.
	aA:=(alphaAngle \\ 360) degreesToRadians .
	bA:=(betaAngle \\ 360) degreesToRadians .
	iR:= innerRadius.
	eR:= externalRadius.
	p1:= (iR*aA cos)@(iR*aA sin) .
	p2:= (iR*bA cos)@(iR*bA sin) .
	p3:= (eR*aA cos)@(eR*aA sin).
	p4:=(eR*bA cos)@(eR*bA sin) .
	pR:=(eR*0 cos) @ (eR*0 sin).
	pQ1:=(eR*(Float pi /4) cos)@(eR*(Float pi /4) sin).
	pB:=(eR*(Float pi /2) cos)@(eR*(Float pi /2) sin).
	pQ2:=(eR*(Float pi * (3/4)) cos)@(eR*(Float pi * (3/4)) sin).
	pL:=(eR*Float pi cos)@(eR*Float pi sin).
	pQ3:=(eR*(Float pi * (5/4)) cos)@(eR*(Float pi * (5/4)) sin).
	pT:=(eR*(Float pi * (3/2)) cos)@(eR*(Float pi * (3/2)) sin).
	pQ4:=(eR*(Float pi * (7/4)) cos)@(eR*(Float pi * (7/4)) sin).
	pCollection add:p1; add: p2; add: p3; add: p4.
	(self includesPoint: pQ1)ifTrue:[pCollection add: pQ1].
	(self includesPoint: pQ2)ifTrue:[pCollection add: pQ2].
	(self includesPoint: pQ3)ifTrue:[pCollection add: pQ3].
	(self includesPoint: pQ4)ifTrue:[pCollection add: pQ4].
	(self includesPoint: pL)ifTrue:[pCollection add: pL].
	(self includesPoint: pR)ifTrue:[pCollection add: pR].
	(self includesPoint: pB)ifTrue:[pCollection add: pB].
	(self includesPoint: pT)ifTrue:[pCollection add: pT].
	topLeft:=pCollection min x @ pCollection max y
	! !

!TRArcShape methodsFor: 'computing' stamp: 'SamirSaleh 1/27/2014 10:47'!
computeWidth
	width:=(self topLeft x - self bottomRight x) abs! !

!TRArcShape methodsFor: 'drawing' stamp: 'SamirSaleh 1/21/2014 10:31'!
drawOn: athensCanvas
	self shouldBeDrawn ifFalse: [ ^ self ].
	athensCanvas pathTransform restoreAfter: [
		athensCanvas
			setPaint: color;
			drawShape: self path.

		self drawStrokeIfNecessaryOn: athensCanvas
		]! !

!TRArcShape methodsFor: 'drawing' stamp: 'SamirSaleh 1/20/2014 17:22'!
drawStrokeIfNecessaryOn: athensCanvas! !

!TRArcShape methodsFor: 'accessing' stamp: 'SamirSaleh 1/24/2014 14:46'!
encompassingRectangle
	^self topLeft corner: self bottomRight! !

!TRArcShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/24/2014 18:13'!
height
	self computeHeight.
	^ height! !

!TRArcShape methodsFor: 'testing' stamp: 'SamirSaleh 1/28/2014 10:50'!
includesPoint: aPoint
	| pDistanceOrigin pointAngle inside aAngle bAngle |
	((alphaAngle) = (betaAngle)) ifTrue:[ ^ false ].
	inside:=true.
	(alphaAngle % 360 = 0 ) 
		ifFalse:[aAngle:=alphaAngle \\  360]
		ifTrue:[aAngle:=alphaAngle].
	(betaAngle % 360 = 0 )
		ifFalse:[bAngle:=betaAngle \\  360]
		ifTrue:[bAngle:= betaAngle].
	pointAngle:= ((aPoint x @ aPoint y negated) angleWith: self center) radiansToDegrees \\  360.
	pDistanceOrigin:=(aPoint squaredDistanceTo: self center) sqrt.
	((pDistanceOrigin < innerRadius) or: (pDistanceOrigin > externalRadius))
	ifTrue:[^false].
	(pointAngle % 360 = 0 )
		ifTrue:[
			(aAngle % 360 = 0) ifTrue:[^true].
			(bAngle % 360 = 0) ifTrue:[^true]].
	(aAngle < bAngle)
	ifTrue:[
		(pointAngle >= aAngle) ifTrue:[ inside:=inside not ].
		(pointAngle <= bAngle) ifTrue:[ inside:=inside not ] ]
	ifFalse:[
		(bAngle = 0) ifTrue:[bAngle:=360].
		(pointAngle <= aAngle) ifTrue:[ inside:=inside not ].
		(pointAngle <= bAngle) ifTrue:[ inside:=inside not ] ].
	^ inside! !

!TRArcShape methodsFor: 'initialization' stamp: 'AlexandreBergel 1/24/2014 18:14'!
initialize
	super initialize.
	center := 0 @ 0.
	height := 0.
	width := 0.
	externalRadius := 0.
	innerRadius := 0.
	alphaAngle := 0.
	betaAngle := 0! !

!TRArcShape methodsFor: 'accessing' stamp: 'SamirSaleh 1/27/2014 09:41'!
position
	^ 0 @ 0
	! !

!TRArcShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/24/2014 18:14'!
topLeft
	self computeTopLeft.
	^ topLeft! !

!TRArcShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/24/2014 18:14'!
width
	self computeWidth.
	^ width! !

TRShape subclass: #TRBezierShape
	instanceVariableNames: 'listOfPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRBezierShape methodsFor: 'hooks' stamp: 'AlexandreBergel 1/13/2014 17:14'!
computePath
	path := self athensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveTo: listOfPoints first.
			
			(2 to: (listOfPoints size -1)) do: [ :index |
				builder 
					curveVia: (listOfPoints at: index) 
					to: (((listOfPoints at: index) + (listOfPoints at: (index + 1))) / 2) asFloatPoint.
			].
			builder lineTo: listOfPoints last.
		]! !

!TRBezierShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/10/2014 14:36'!
drawOn: athensCanvas

	athensCanvas pathTransform restoreAfter: [
			
		athensCanvas
			setStrokePaint: strokePaint;
			drawShape: self path  ]! !

!TRBezierShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/10/2014 17:09'!
encompassingRectangle
	^ Rectangle encompassing: listOfPoints! !

!TRBezierShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/22/2014 15:46'!
initialize
	super initialize.
	listOfPoints := Array with: 0 @ 0 with: 0 @ 0! !

!TRBezierShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/16/2014 15:31'!
points
	^ listOfPoints copy! !

!TRBezierShape methodsFor: 'public' stamp: 'AlexandreBergel 1/9/2014 23:06'!
points: points
	listOfPoints := points! !

!TRBezierShape methodsFor: 'public' stamp: 'AlexandreBergel 1/12/2014 16:58'!
points: aListOfPoints color: aColor

	self points: aListOfPoints.
	self strokePaint: aColor.
	self color: aColor! !

!TRBezierShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/22/2014 16:28'!
position
	^ self encompassingRectangle center! !

!TRBezierShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/16/2014 15:28'!
translateBy: aPoint
	listOfPoints := listOfPoints collect: [ :p | p:=p+aPoint ]
! !

TRShape subclass: #TRComposedShape
	instanceVariableNames: 'shape1 shape2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRComposedShape class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/13/2014 17:13'!
with: s1 with: s2
	^ self new shape1: s1; shape2: s2; yourself! !

!TRComposedShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/13/2014 17:32'!
addCallback: aCallback
	shape1 addCallback: aCallback.
	shape2 addCallback: aCallback! !

!TRComposedShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/13/2014 18:52'!
addedIn: aCanvas
	aCanvas addShape: shape1.
	aCanvas addShape: shape2.
! !

!TRComposedShape methodsFor: 'events' stamp: 'AlexandreBergel 1/13/2014 17:36'!
announce: anEvent
	shape1 announce: anEvent.
	shape2 announce: anEvent.! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 19:24'!
announcer
	self error: 'should not be called'! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 19:04'!
announcer: anAnnouncer
	shape1 announcer: anAnnouncer.
	shape2 announcer: anAnnouncer.! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 17:32'!
canvas: aSurface
	"aCanvas is a AthensCairoSurface"
	shape1 canvas: aSurface.
	shape2 canvas: aSurface! !

!TRComposedShape methodsFor: 'hooks' stamp: 'AlexandreBergel 1/13/2014 17:15'!
computePath
	shape1 computePath.
	shape2 computePath! !

!TRComposedShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/13/2014 17:15'!
drawOn: athensCanvas

	shape1 drawOn: athensCanvas.
	shape2 drawOn: athensCanvas! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 19:32'!
element
	^ shape1 element! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 19:12'!
element: anObject
	"anObject could be any kind of object. A shape has a reference to an object, typically the object odel that the trachel shape represents"
	shape1 element: anObject.
	shape2 element: anObject.! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 17:14'!
encompassingRectangle
	^ Rectangle merging: (Array with: shape1 encompassingRectangle with: shape2 encompassingRectangle)! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/14/2014 09:25'!
height
	^ self encompassingRectangle height! !

!TRComposedShape methodsFor: 'testing' stamp: 'AlexandreBergel 1/13/2014 17:36'!
includesPoint: aPoint
	^ (shape1 includesPoint: aPoint) or: [ shape2 includesPoint: aPoint ]! !

!TRComposedShape methodsFor: 'initialization' stamp: 'AlexandreBergel 1/22/2014 15:47'!
initialize
	super initialize.
	shape1 := TRNoShape instance.
	shape2 := TRNoShape instance.! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/22/2014 15:47'!
position
	^ (shape1 position + shape2 position) / 2! !

!TRComposedShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/14/2014 15:25'!
remove
	shape1 remove.
	shape2 remove! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 17:13'!
shape1
	^ shape1! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 17:13'!
shape1: aTRShape
	shape1 := aTRShape! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 17:13'!
shape2
	^ shape2! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/13/2014 17:13'!
shape2: aTRShape
	shape2 := aTRShape! !

!TRComposedShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/15/2014 16:19'!
signalUpdate
	shape1 signalUpdate! !

!TRComposedShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/23/2014 14:32'!
translateTo: aPoint
	shape1 translateTo: aPoint.
	shape2 translateTo: aPoint.! !

!TRComposedShape methodsFor: 'events' stamp: 'JurajKubelka 1/27/2014 10:22'!
when: event do: aBlock
	shape1 announcer when: event do: aBlock.
	shape2 announcer when: event do: aBlock! !

!TRComposedShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/14/2014 09:26'!
width
	^ self encompassingRectangle width! !

TRShape subclass: #TRLabelShape
	instanceVariableNames: 'text font fontSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRLabelShape class methodsFor: 'configuration' stamp: 'AlexandreBergel 1/1/2014 18:35'!
defaultFontSize
	^ 12! !

!TRLabelShape class methodsFor: 'configuration' stamp: 'AlexandreBergel 1/1/2014 19:10'!
getFontForSize: aNumber

	^ LogicalFont familyName: 'Arial' pointSize: aNumber! !

!TRLabelShape class methodsFor: 'configuration' stamp: 'AlexandreBergel 1/1/2014 18:50'!
widthOfText: aString fontSize: aNumber
	^ (self getFontForSize: aNumber) widthOfString: aString! !

!TRLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 13:39'!
ascent
	^ font ascent! !

!TRLabelShape methodsFor: 'initialization' stamp: 'AlexandreBergel 1/1/2014 18:35'!
defaultFontSize
	^ self class defaultFontSize! !

!TRLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 13:39'!
descent
	^ font descent! !

!TRLabelShape methodsFor: 'drawing' stamp: 'JurajKubelka 1/20/2014 15:51'!
drawOn: athensCanvas
	"self drawEncompassingRectangleOn: athensCanvas color: (Color gray alpha: 0.1)."
	athensCanvas pathTransform restoreAfter: [
		"Draw the label centered on the position"
		athensCanvas pathTransform
			translateBy: position asIntegerPoint;
			translateBy: ((self width / -2) @ (self height / 2)) asIntegerPoint;
			translateBy: (0 @ self descent negated) asIntegerPoint.
			"scaleBy: (fontSize / 10)."
		athensCanvas setFont: font.
		athensCanvas setPaint: color.
		athensCanvas drawString: text
		]! !

!TRLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 12:21'!
encompassingRectangle
	| w h |
	w := self width.
	h := self height.
	^ position + ((w / -2) @ (h / -2)) extent: (w @ h)! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 21:50'!
font
	^ font! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/1/2014 18:09'!
fontSize
	^ fontSize! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/1/2014 18:49'!
fontSize: aNumber
	font := self class getFontForSize: aNumber.
	fontSize := aNumber! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 22:06'!
height
	^ font height! !

!TRLabelShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 11:03'!
includesPoint: aPoint
	^ self encompassingRectangle containsPoint: aPoint! !

!TRLabelShape methodsFor: 'initialization' stamp: 'AlexandreBergel 1/28/2014 17:40'!
initialize
	super initialize.
	text := 'Trachel label'.
	self fontSize: self defaultFontSize! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 14:17'!
leftPosition: aPosition
	self position: (aPosition - ((self width / -2) @ 0) ) asIntegerPoint! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/17/2014 11:01'!
position
	^ position! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 22:04'!
position: aPosition
	position := aPosition! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/1/2014 18:08'!
text
	^ text! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 22:03'!
text: aText
	text := aText! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 22:04'!
text: aText on: aPosition
	self text: aText.
	self position: aPosition! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 15:04'!
translateBy: step
	position := position + step.! !

!TRLabelShape methodsFor: 'public' stamp: 'AlexandreBergel 1/1/2014 15:04'!
translateTo: aPoint
	position := aPoint! !

!TRLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/1/2014 18:54'!
width
	^ (font widthOfString: text)! !

TRLabelShape subclass: #TRRotatedLabelShape
	instanceVariableNames: 'angle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRRotatedLabelShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/17/2014 15:36'!
angleInDegree: anAngle
	angle := anAngle! !

!TRRotatedLabelShape methodsFor: 'drawing' stamp: 'JurajKubelka 1/20/2014 15:51'!
drawOn: athensCanvas
	"self drawEncompassingRectangleOn: athensCanvas color: (Color green alpha: 0.1)."
	athensCanvas pathTransform restoreAfter: [
		"Draw the label centered on the position"
		athensCanvas pathTransform 
			translateBy: position asIntegerPoint;
			rotateByDegrees: angle;
			translateBy: ((self notRotatedWidth / -2) @ (self notRotatedHeight / 2)) asIntegerPoint;
			translateBy: (0 @ font descent negated) asIntegerPoint.
		athensCanvas setFont: font.
		athensCanvas setPaint: color.
		athensCanvas drawString: text
	].! !

!TRRotatedLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 15:49'!
encompassingRectangle

	| w h transformation translation notRotatedRectangle rotatedRectangle topLeft topRight bottomRight bottomLeft |
	w := self notRotatedWidth.
	h := self notRotatedHeight.
	transformation := AthensAffineTransform new 
						rotateByDegrees: angle;
						translateBy: ((w / -2) @ (h / 2)) asIntegerPoint.
	notRotatedRectangle  := 0 @ h negated corner: (w @ 0).
	topLeft := transformation transform: notRotatedRectangle topLeft.
	topRight := transformation transform: notRotatedRectangle topRight.
	bottomRight := transformation transform: notRotatedRectangle bottomRight.
	bottomLeft := transformation transform: notRotatedRectangle bottomLeft.
	rotatedRectangle := 
		({ topLeft x. topRight x. bottomRight x. bottomLeft x} min @
		{ topLeft y. topRight y. bottomRight y. bottomLeft y} min) corner:
		({ topLeft x. topRight x. bottomRight x. bottomLeft x} max @
		{ topLeft y. topRight y. bottomRight y. bottomLeft y} max).
	translation := AthensAffineTransform new
					translateBy: position asIntegerPoint.
	^ (translation transform: rotatedRectangle topLeft)
	 	corner: (translation transform: rotatedRectangle bottomRight)
	! !

!TRRotatedLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 15:47'!
height

	^ self encompassingRectangle height! !

!TRRotatedLabelShape methodsFor: 'initialization' stamp: 'AlexandreBergel 1/17/2014 15:35'!
initialize
	super initialize.
	angle := 0.! !

!TRRotatedLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 15:49'!
notRotatedHeight
	^ font height! !

!TRRotatedLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 15:49'!
notRotatedWidth
	^ (font widthOfString: text)! !

!TRRotatedLabelShape methodsFor: 'accessing' stamp: 'JurajKubelka 1/20/2014 15:47'!
width

	^ self encompassingRectangle width! !

TRShape subclass: #TRLineShape
	instanceVariableNames: 'from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRLineShape methodsFor: 'hooks' stamp: 'AlexandreBergel 12/26/2013 10:35'!
computePath
	path := self athensCanvas
		createPath: [ :builder | 
			builder
				absolute;
				moveTo: from;
				lineTo: to ]! !

!TRLineShape methodsFor: 'drawing' stamp: 'AlexandreBergel 1/19/2014 17:38'!
drawOn: athensCanvas

	athensCanvas pathTransform restoreAfter: [
		athensCanvas
			setStrokePaint: strokePaint;
			drawShape: self path  ]
		"(athensCanvas setStrokePaint: strokePaint) width: (1.5 / (scale )) asInteger."! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/28/2013 15:22'!
encompassingRectangle
	^ Rectangle encompassing: (Array with: from with: to)! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:18'!
from
	"Return a point"
	^ from! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:18'!
from: aPoint
	from := aPoint! !

!TRLineShape methodsFor: 'public' stamp: 'AlexandreBergel 12/24/2013 16:33'!
from: aPoint to: anotherPoint
	from := aPoint.
	to := anotherPoint.
! !

!TRLineShape methodsFor: 'public' stamp: 'AlexandreBergel 12/14/2013 18:46'!
from: aPoint to: anotherPoint color: aColor

	self from: aPoint to: anotherPoint.
	self strokePaint: aColor.
	self color: aColor! !

!TRLineShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/15/2013 19:16'!
includesPoint: aPoint
	^ aPoint onLineFrom: from to: to within: 3! !

!TRLineShape methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/22/2014 15:48'!
initialize
	super initialize.
	from := 0 @ 0.
	to := 0 @ 0! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/22/2014 16:28'!
position
	^ ((from + to) / 2)! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 19:18'!
to
	"Return a point"
	^ to! !

!TRLineShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/17/2013 16:52'!
to: aPoint
	to := aPoint! !

!TRLineShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:35'!
translateBy: aPoint
	from := from + aPoint.
	to := to + aPoint! !

TRShape subclass: #TRNoShape
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

TRNoShape class
	instanceVariableNames: 'instance'!

!TRNoShape commentStamp: 'TudorGirba 1/4/2014 13:21' prior: 0!
This implements the Null Pattern for a Trachel shape.!

TRNoShape class
	instanceVariableNames: 'instance'!

!TRNoShape class methodsFor: 'as yet unclassified' stamp: 'AlexandreBergel 1/21/2014 17:19'!
instance
	instance ifNil: [ instance := self new ].
	^ instance! !

!TRNoShape methodsFor: 'accessing' stamp: 'TudorGirba 1/4/2014 13:15'!
center
	0@0! !

!TRNoShape methodsFor: 'hooks' stamp: 'TudorGirba 1/4/2014 13:15'!
computePath
	"do nothing"! !

!TRNoShape methodsFor: 'drawing' stamp: 'TudorGirba 1/4/2014 13:19'!
drawOn: athensCanvas
	"do nothing"! !

!TRNoShape methodsFor: 'accessing' stamp: 'TudorGirba 1/4/2014 13:14'!
encompassingRectangle
	^ 0@0 corner: 0@0! !

!TRNoShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/5/2014 12:09'!
height
	^ 0! !

!TRNoShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/22/2014 15:48'!
position
	^ 0 @ 0! !

!TRNoShape methodsFor: 'actions' stamp: 'TudorGirba 1/4/2014 13:15'!
translateBy: aPoint
	"do nothing"! !

!TRNoShape methodsFor: 'actions' stamp: 'TudorGirba 1/4/2014 13:15'!
translateTo: aPoint
	"do nothing"! !

!TRNoShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/5/2014 12:09'!
width
	^ 0! !

TRShape subclass: #TRSVGPath
	instanceVariableNames: 'borderWidth fillColor scale paths pathPoints points curves rotationAngle rectangle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/27/2014 12:14'!
angle
	^ rotationAngle! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/20/2014 16:48'!
borderColor: aColor
	self strokeColor: aColor! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/21/2014 10:40'!
borderWidth: width
	borderWidth := width.! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/27/2014 15:29'!
center
	^ rectangle center! !

!TRSVGPath methodsFor: 'accessing' stamp: 'AlexandreBergel 1/28/2014 19:26'!
color
	^ fillColor! !

!TRSVGPath methodsFor: 'accessing' stamp: 'AlexandreBergel 1/28/2014 18:47'!
color: aColor
	^ self fillColor: aColor! !

!TRSVGPath methodsFor: 'hooks' stamp: 'SergioMaass 1/22/2014 23:58'!
computePath
	path := (self athensCanvas createPath: [ :builder |
			builder absolute.
			curves do: [ :c |
				c key = 'm' ifTrue: [ builder moveTo: (c value at: 1) ].
				c key = 'l' ifTrue: [ builder lineTo: (c value at: 1) ].
				c key = 'c' ifTrue: [ builder curveVia: (c value at: 2) and: (c value at: 3) to: (c value at: 1) ].
				c key = 'q' ifTrue: [ builder curveVia: (c value at: 2) to: (c value at: 1) ]. ]
		])! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/22/2014 14:55'!
curves
	^ curves! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/27/2014 15:38'!
curves: someCurves
	curves := someCurves.
	paths := self separateSubPaths: someCurves.
	pathPoints := OrderedCollection new.
	paths do: [ :p |
		|pts|
		pts := OrderedCollection new.
		p do: [ :curve |
			curve key = 'm' ifFalse: [pts add: (curve value at: 1) * scale]
		].
		pathPoints add: pts.
	].
	points := pathPoints inject: #() into: [ :acc :next | acc, next ].
	position := self encompassingRectangle center.! !

!TRSVGPath methodsFor: 'drawing' stamp: 'SergioMaass 1/26/2014 23:19'!
drawOn: aCanvas
	| pt |
	pt := aCanvas pathTransform.
	pt restoreAfter: [
		pt scaleBy: scale.
		aCanvas setShape: self path.
		fillColor isNotNil ifTrue: [ aCanvas setPaint: fillColor; draw ].
		(aCanvas setStrokePaint: strokePaint) width: borderWidth.
		aCanvas draw
	]! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/27/2014 15:28'!
encompassingRectangle
	rectangle := Rectangle encompassing: points.
	^ rectangle! !

!TRSVGPath methodsFor: 'drawing' stamp: 'SergioMaass 1/22/2014 11:39'!
figure: aCanvas
	^ aCanvas createPath: [:builder |
		builder
			absolute;
			moveTo: 100@200 ;
			curveVia: 100@100 and: 250@100 to: 250@ 200;
			curveVia: 250@300 and: 400@300 to: 400@200 ]
	! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/20/2014 16:40'!
fillColor: aColor
	fillColor := aColor! !

!TRSVGPath methodsFor: 'testing' stamp: 'SergioMaass 1/21/2014 16:41'!
includesPoint: aPoint
	|apt|
	apt := AthensPolygonTester new.
	^ pathPoints inject: false into: [ :res :p |
		res or: ((apt polygon: p) includesPoint: aPoint)
	]! !

!TRSVGPath methodsFor: 'initialization' stamp: 'AlexandreBergel 1/28/2014 19:28'!
initialize
	super initialize.
	curves := #().
	paths := #().
	pathPoints := #().
	points := #().
	rotationAngle := 0.
	rectangle := Rectangle origin: 0@0 corner: 0@0.! !

!TRSVGPath methodsFor: 'actions' stamp: 'SergioMaass 1/27/2014 12:20'!
rotateBy: anAngle
	|center|
	center := self center / scale.
	
	self curves: (self curves collect: [ :c |
		c key -> (c value collect: [ :p |
			| pp cosTheta sinTheta |
			pp := p - center.
			cosTheta := anAngle cos.
			sinTheta := anAngle sin.
			(pp x * cosTheta - (pp y * sinTheta)) @ (pp x * sinTheta + (pp y * cosTheta))
			+ center.
		])]).
	
	rotationAngle := rotationAngle + anAngle.
	^ rotationAngle.! !

!TRSVGPath methodsFor: 'actions' stamp: 'SergioMaass 1/27/2014 12:21'!
rotateTo: anAngle
	^ self rotateBy: anAngle - rotationAngle.! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/21/2014 16:02'!
scale
	^ scale! !

!TRSVGPath methodsFor: 'accessing' stamp: 'SergioMaass 1/20/2014 16:59'!
scale: aNumber
	scale := aNumber! !

!TRSVGPath methodsFor: 'hooks' stamp: 'SergioMaass 1/22/2014 23:45'!
separateSubPaths: someCurves
	| subPath |
	paths := OrderedCollection new.
	subPath := OrderedCollection new.
	
	someCurves do: [ :c |
		c key = 'm' ifTrue: [ paths add: subPath. subPath:= OrderedCollection new ].
		subPath add: c.
	].
	paths add: subPath.
	^ paths! !

!TRSVGPath methodsFor: 'actions' stamp: 'AlexandreBergel 1/29/2014 18:31'!
translateTo: aPoint
	| disp |
	disp := (aPoint - position) / scale.
	super translateTo: aPoint.
	self curves: (self curves collect: [ :c |
		c key -> (c value collect: [ :p | p + disp ])]).
	rectangle := rectangle translateBy: disp.! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/29/2013 10:27'!
addCallback: aCallback
	callbacks ifNil: [ callbacks := OrderedCollection new ].
	callbacks add: aCallback! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/13/2014 17:51'!
addedIn: aCanvas
	aCanvas privateAdd: self.
	self canvas: aCanvas! !

!TRShape methodsFor: 'events' stamp: 'AlexandreBergel 12/15/2013 23:11'!
announce: anEvent
	self announcer announce: anEvent! !

!TRShape methodsFor: 'events' stamp: 'AlexandreBergel 12/15/2013 19:23'!
announcer
	announcer isNil ifTrue: [ announcer := Announcer new ].
	^ announcer! !

!TRShape methodsFor: 'events' stamp: 'AlexandreBergel 12/16/2013 09:44'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/25/2013 09:26'!
athensCanvas
	^ self canvas athensCanvas! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/25/2013 09:25'!
canvas
	"Return a Trachel Canvas"
	^ canvas! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 17:47'!
canvas: aSurface
	"aCanvas is a AthensCairoSurface"
	canvas := aSurface! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 10:01'!
center
	self subclassResponsibility! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:10'!
color
	
	^ color! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:10'!
color: aColor
	
	color := aColor! !

!TRShape methodsFor: 'hooks' stamp: 'AlexandreBergel 12/24/2013 17:02'!
computePath
	self subclassResponsibility! !

!TRShape methodsFor: 'drawing' stamp: 'JurajKubelka 1/20/2014 13:00'!
drawEncompassingRectangleOn: athensCanvas
	"debugging purpose"
	self drawEncompassingRectangleOn: athensCanvas color: (Color blue alpha: 0.2)! !

!TRShape methodsFor: 'drawing' stamp: 'JurajKubelka 1/20/2014 13:00'!
drawEncompassingRectangleOn: athensCanvas color: aColor
	"debugging purpose"
 	| r tpath |
	r := self encompassingRectangle.
	tpath := athensCanvas
				createPath: [ :builder | 
					builder absolute; moveTo: r topLeft; lineTo: r topRight; 
							lineTo: r bottomRight; lineTo: r bottomLeft; lineTo: r topLeft ].
	athensCanvas pathTransform
		restoreAfter: [ 
			athensCanvas
				setPaint: aColor;
				drawShape: tpath ].! !

!TRShape methodsFor: 'drawing' stamp: 'AlexandreBergel 12/29/2013 09:57'!
drawOn: athensCanvas

	athensCanvas pathTransform restoreAfter: [
		athensCanvas
			setPaint: color;
			drawShape: self path.
			
		athensCanvas
			setStrokePaint: strokePaint;
			drawShape: self path  ]! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 22:52'!
element
	"A shape has a reference to an object, typically the object odel that the trachel shape represents"
	^ element ! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/16/2013 22:52'!
element: anObject
	"anObject could be any kind of object. A shape has a reference to an object, typically the object odel that the trachel shape represents"
	element := anObject! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/28/2013 15:21'!
encompassingRectangle
	self subclassReponsibility! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/14/2014 09:39'!
extent: anExtent
	"do nothing here. This method may be overriden for particular behavior"! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/29/2013 10:25'!
hasCallback
	^ callbacks notNil! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/15/2013 19:12'!
includesPoint: aPoint
	^ false! !

!TRShape methodsFor: 'initialization' stamp: 'AlexandreBergel 1/28/2014 17:40'!
initialize
	super initialize.
	color := Color gray.
	strokePaint := Color black.	 
	position := 0 @ 0.! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:00'!
isFixed

	^ self canvas fixedShapes includes: self! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 1/22/2014 14:27'!
isInACanvas
	^ canvas notNil! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:09'!
isMovable

	^ self canvas isMovableShape: self! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 12/31/2013 10:00'!
isNotFixed

	^ self isMovable! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/24/2013 17:02'!
path
	path ifNil: [ self computePath ].
	^ path! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 1/28/2014 17:42'!
position
	^ position! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/8/2014 15:18'!
pushBack
	canvas pushBack: self
	
	! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/8/2014 15:18'!
pushFront
	canvas pushFront: self
	
	! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/22/2014 14:28'!
remove
	"Return true if I am in a canvas"
	
	"Do nothing if the line is not already in a canvas"
	self isInACanvas ifFalse: [ ^ self ]. 
	canvas removeShape: self! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/24/2013 17:07'!
resetPath
	path := nil! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/17/2014 10:47'!
setAsFixed
	canvas ifNil: [ self error: 'You should first insert the shape in the canvas before setting it as fixed'. ].
	canvas setAsFixed: self.
	! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 12/31/2013 09:58'!
setAsNotFixed
	canvas removeFixedShape: self.
	canvas addShape: self.! !

!TRShape methodsFor: 'testing' stamp: 'AlexandreBergel 1/1/2014 15:52'!
shouldBeDrawn
	^ true! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/5/2014 18:23'!
signalUpdate
	"Refresh the canvas"
	canvas ifNil: [ ^ self ].
	canvas signalUpdate! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:48'!
strokePaint
	"Return the color of the border line"
	^ strokePaint! !

!TRShape methodsFor: 'accessing' stamp: 'AlexandreBergel 12/14/2013 10:49'!
strokePaint: aColor
	"set the color of the border line"
	strokePaint := aColor! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/28/2014 17:40'!
translateBy: aPoint
	position := position + aPoint! !

!TRShape methodsFor: 'actions' stamp: 'AlexandreBergel 1/28/2014 17:40'!
translateTo: aPoint
	position := aPoint! !

!TRShape methodsFor: 'events' stamp: 'JurajKubelka 1/27/2014 10:22'!
when: event do: aBlock
	self announcer when: event do: aBlock! !

TRObject subclass: #TRTranslationCallback
	instanceVariableNames: 'block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Core'!

!TRTranslationCallback class methodsFor: 'public' stamp: 'AlexandreBergel 12/29/2013 10:37'!
block: aBlock
	^ self new block: aBlock! !

!TRTranslationCallback methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 10:22'!
block
	^ block
	! !

!TRTranslationCallback methodsFor: 'accessing' stamp: 'AlexandreBergel 12/29/2013 10:43'!
block: aOneArgBlock
	"aThreeArgBlock follows the pattern [ :shape | ... ]"
	block := aOneArgBlock! !

!TRTranslationCallback methodsFor: 'initialization' stamp: 'AlexandreBergel 12/29/2013 10:57'!
initialize
	super initialize.
	block := [ :shape :step | ]! !

Object subclass: #VIAnimatedValue
	instanceVariableNames: 'evaluator timer finishCallback'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Viva'!

!VIAnimatedValue commentStamp: '<historical>' prior: 0!
A VIAnimatedValue is A representation of some property value animated in certain kind of time.

The main idea is you can change some property in some time.
in evaluator, recibe a block to handle as f(t).

!

!VIAnimatedValue methodsFor: 'grouping' stamp: 'AlexandreBergel 1/12/2014 16:58'!
+ aVIAnimatedValue 
	self error: 'Not supported yet'
"	^VIAnimatedGroup new 
		add:self;
		add:aVIAnimatedValue."! !

!VIAnimatedValue methodsFor: 'secuence' stamp: 'AlexandreBergel 1/12/2014 16:58'!
, aVIAnimatedValue 
	self error: 'Not yet supported'
"	^VIAnimatedSecuence new
		add:self;
		add:aVIAnimatedValue.
	"! !

!VIAnimatedValue methodsFor: 'actions' stamp: 'MarianoVicente 9/15/2013 23:19'!
animatedPoint: aPoints
	| pnts |
	pnts := aPoints asSortedCollection.
	self evaluator: [ :t |
		| inicial fin |
		inicial := nil.
		fin := nil.
		
		pnts do:[ :e | 
	 	    e key <= t 
			ifTrue: [ inicial:= e ] 
			ifFalse:[fin isNil ifTrue:[fin:= e] ]
	 	].
		inicial value interpolateTo: fin value at: t.
	 ].! !

!VIAnimatedValue methodsFor: 'actions' stamp: 'MarianoVicente 9/15/2013 23:33'!
animatedValue: values
	| val fraction currentValue |
	fraction := 1.0 / (values size -1).
	currentValue:= 0.
	self animatedPoint: 
		(values collect: [ :e | val := currentValue -> e. currentValue:= currentValue + fraction. val]).! !

!VIAnimatedValue methodsFor: 'accessing' stamp: 'MarianoVicente 9/16/2013 00:01'!
animationDidFinish
	timer value = 1.0
		ifTrue: [ finishCallback value ].! !

!VIAnimatedValue methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 05:34'!
evaluator
	
	^ evaluator! !

!VIAnimatedValue methodsFor: 'accessing' stamp: 'MarianoVicente 9/15/2013 23:25'!
evaluator: anObject
	evaluator := anObject! !

!VIAnimatedValue methodsFor: 'accessing' stamp: 'MarianoVicente 9/16/2013 00:02'!
finishCallback: anObject
	finishCallback := anObject! !

!VIAnimatedValue methodsFor: 'testing' stamp: 'AlexandreBergel 1/1/2014 20:05'!
hasCompleted
	^ timer value = 1.0! !

!VIAnimatedValue methodsFor: 'actions' stamp: 'MarianoVicente 9/15/2013 23:22'!
initialize
	super initialize.
	timer := VITimer new.! !

!VIAnimatedValue methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 15:09'!
reset
	timer reset! !

!VIAnimatedValue methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 15:08'!
start
	timer start! !

!VIAnimatedValue methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 15:08'!
stop
	timer stop! !

!VIAnimatedValue methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 15:08'!
timer
	^ timer! !

!VIAnimatedValue methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 15:08'!
timer: anObject
	timer := anObject! !

!VIAnimatedValue methodsFor: 'accessing' stamp: 'MarianoVicente 9/15/2013 23:59'!
value
	" evaluate the value of receiver at current time "
		self animationDidFinish.
	^ evaluator value: timer value! !

Object subclass: #VIBasicTimer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Viva'!

!VIBasicTimer commentStamp: '<historical>' prior: 0!
I represent an abstract behavior of timer.

We defining a cycle length (cycleLength) for timer, expressed in seconds.
The value of timer (#value) then can be observed over time
as a number which goes from 0 to 1 and then stops.
The moment when value reaches 1 is at time equal to timer's cycle length passed from the moment we started it.
 
Timers can be one-shot or repeating.
One-shot timer stops after elapsing its cycle length, reaching value = 1.
The repeating timer keeps running repeating the cycle over and over again, and its value goes from 0 to 1 , then drops to 0 and goes again to 1 and again...
Additionally to #value, there is #elapsedCycles , which can be used to tell, how many cycles are passed since timer's first start.

Timer can be stopped (paused), resumed or fully reset. The timer value & elapsed cycles is changing only when timer active.!

!VIBasicTimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:10'!
cycleLength
	"answer the timer cycle length, in seconds"
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:10'!
cycleLength: aNumber
	"Set the timer cycle length, in seconds"
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:10'!
elapsedCycles
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'testing' stamp: 'IgorStasenko 6/1/2013 08:09'!
isActive
	"Answer whether timer currently active or not"
	
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'testing' stamp: 'IgorStasenko 6/1/2013 08:20'!
isRepeating
	"Answer whether timer is one-shot or repeating"
	
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'converting' stamp: 'AlexandreBergel 1/12/2014 16:59'!
mirrored
	self error: 'not yet supported'
	"^ VIMirrorRepeatTimer on: self"! !

!VIBasicTimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:20'!
noRepeat
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 08:14'!
pause
	"synonym to stop"
	
	^ self stop! !

!VIBasicTimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:20'!
repeat
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 08:14'!
reset
	"Reset the timer"
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 08:14'!
resume
	"synonym to start"
	
	^ self start! !

!VIBasicTimer methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 08:13'!
start
	"Start the timer, after which its value change can be observed over time"
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 08:14'!
stop

	"Stop the timer"
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:12'!
value
	"answer the current value of timer"
	self subclassResponsibility ! !

!VIBasicTimer methodsFor: 'converting' stamp: 'AlexandreBergel 1/12/2014 16:59'!
withBias: aWeight
	self error: 'not yet supported'
	"^ (VIBiasedTimerModifier on: self) weight: aWeight"! !

VIBasicTimer subclass: #VITimer
	instanceVariableNames: 'active cycleLength repeat startTime elapsedCycles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Trachel-Viva'!

!VITimer commentStamp: '<historical>' prior: 0!
I am using system time to implement a timer.

See my supeclass (and my method comments) for more details.!

!VITimer methodsFor: 'private' stamp: 'IgorStasenko 6/1/2013 06:30'!
currentTime
	^ Time millisecondClockValue ! !

!VITimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 06:23'!
cycleLength
	^ cycleLength! !

!VITimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 06:23'!
cycleLength: anObject
	cycleLength := anObject! !

!VITimer methodsFor: 'private' stamp: 'IgorStasenko 6/1/2013 06:34'!
cycleLengthScale
	"cycle length are given in seconds,
	while internally we using milliseconds to calculate time difference, hence scale is equal to 1000"
	
	^ 1000.0! !

!VITimer methodsFor: 'accessing' stamp: 'MarianoVicente 9/13/2013 19:57'!
elapsedCycles

	"Answer the total number of cycles passed during which timer was active since its first activation.
	For example, if timer's cycle length is 2 seconds, and its total active time is 3.5 seconds,
	a total number of elapsed cycles will be 3.5 / 2 = 1.75.
	
	Suspending timer (stopping then starting after a while) does not contributes to elapsedCycles, e.g.:
	for fresh timer with 4 seconds cycle, we start timer, run for 0.5 seconds, then stop it,
	wait for arbitrary amound of time, then start it again and run for extra 0.5 seconds,
	the measures elapsedCycles value will be equal to 1/4 .
	
	As you may guess, for non-repeating timer, the elapsed cycles value can never be
	greater than 1 (timer always stops at 1 and never advanced since then).
	
	Resetting timer sets elapsed cycles to zero.
	" 
	
	| cycles |
	active ifFalse: [  ^ elapsedCycles  ].

	cycles := elapsedCycles + (self elapsedTime / cycleLength).

	(repeat value not and: [ cycles  >= 1.0 ]) ifTrue: [ 
		active := false.
		^ elapsedCycles := 1.0
	].	
	^ cycles! !

!VITimer methodsFor: 'private' stamp: 'IgorStasenko 6/1/2013 06:31'!
elapsedTime
	^  (self currentTime - startTime) / self cycleLengthScale
! !

!VITimer methodsFor: 'initialize-release' stamp: 'IgorStasenko 6/1/2013 07:25'!
initialize
	"defaults"
	repeat := false.
	cycleLength := 1.0.
	
	self reset.! !

!VITimer methodsFor: 'testing' stamp: 'IgorStasenko 6/1/2013 06:51'!
isActive
	self value. "making sure it will stop if cycle time elapsed for non-repeating timer"
	
	^ active! !

!VITimer methodsFor: 'testing' stamp: 'IgorStasenko 6/1/2013 08:19'!
isRepeating
	^ repeat ! !

!VITimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:18'!
noRepeat
	"make receiver to be non-repeating timer (stops once first cycle elapses)"
	repeat := false ! !

!VITimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 08:17'!
repeat
	"make receiver to be a repeating timer"
	repeat := true ! !

!VITimer methodsFor: 'accessing' stamp: 'MarianoVicente 9/13/2013 20:09'!
repeat: veces
	"make receiver to be a repeating timer"
	repeat := [elapsedCycles < veces ]! !

!VITimer methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 07:42'!
reset
	"Reset the timer: stopping it, if it was active,
	and setting its internal elapsed cycles counter to Zero "
	
	active := false.
	elapsedCycles := 0.0.
! !

!VITimer methodsFor: 'actions' stamp: 'IgorStasenko 6/1/2013 06:45'!
start
	"Activate (start or resume) the timer.
	Ignored if timer is already active"
	
	active ifFalse: [
		startTime := self currentTime.
		active := true.
	].
! !

!VITimer methodsFor: 'accessing' stamp: 'nnydjesus 6/24/2013 16:13'!
startTime
	^ startTime! !

!VITimer methodsFor: 'actions' stamp: 'MarianoVicente 9/15/2013 23:21'!
stop
	"Stop/pausing the timer "
	
	active ifTrue: [
		elapsedCycles := self elapsedCycles.
		active := false.
	].
! !

!VITimer methodsFor: 'accessing' stamp: 'IgorStasenko 6/1/2013 07:58'!
value
	"Answer the current timer value.
	The value is normalized in range between 0 and 1
	relative to cycle length and elapsed time since timer started, e.g.
	if cycle length is 0:01:00
	and elapsed time is 0:00:30,
	the value will be 0.5 (half of a minute).
	For repeating timer the value goes from 0 to 1, then repeats over.
	For non-repeating timer, when value reaching 1, timer stops.
	If timer is inactive, answer the phase of cycle where it was stopped.
	"
	| cycles |

	active ifFalse: [ 
		^ repeat 
			ifTrue: [ elapsedCycles fractionPart ]
			ifFalse: [ elapsedCycles min: 1.0 ]
	].

	cycles := self elapsedCycles.
	active ifFalse: [
		 "non-repeating timer was stopped" 
		^ 1.0
	].	
	 
	^  cycles fractionPart.
! !
